#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars import _CustomHinduHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import SEP, OCT, NOV, DEC
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
    ALL_TO_NEAREST_MON,
)


class Fiji(
    ObservedHolidayBase,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
):
    """Fiji holidays.

    References:
        * <https://web.archive.org/web/20250421221329/https://www.laws.gov.fj/Acts/DisplayAct/2910>
        * <https://web.archive.org/web/20250421155009/https://laws.gov.fj/LawsAsMade>
        * <https://web.archive.org/web/20250318092311/https://www.fiji.gov.fj/About-Fiji/Public-Holidays>
        * <https://web.archive.org/web/20250413132828/https://www.timeanddate.com/holidays/fiji/>
        * <https://en.wikipedia.org/wiki/List_of_festivals_in_Fiji>
        * <https://web.archive.org/web/20250427183802/https://www.rnz.co.nz/international/pacific-news/249514/new-public-holiday-for-fiji>
        * <https://web.archive.org/web/20250427183723/https://www.fijitimes.com.fj/constitution-day-public-holiday-removed-cabinet/>
        * <https://web.archive.org/web/20150626160414/http://fijivillage.com:80/news/National-Sports-Day-celebrated-5krs29/>
        * <https://web.archive.org/web/20240624074852/https://www.fijivillage.com/news/Cabinet-approves-Ratu-Sir-Lala-Sukuna-Day-and-Girmit-Day-and-removes-Constitution-Day-as-a-public-holiday-f48r5x/>

    Official Fiji Public Holidays Calendar:
         * [2016](https://web.archive.org/web/20160520212352/http://www.fiji.gov.fj:80/Media-Center/Press-Releases/GOVERNMENT-APPROVES-2016-PUBLIC-HOLIDAYS.aspx?)
         * [2017](https://web.archive.org/web/20250319202817/https://www.fiji.gov.fj/Media-Centre/News/GOVERNMENT-APPROVES-2017-PUBLIC-HOLIDAYS)
         * [2018](https://web.archive.org/web/20180727205733/http://www.employment.gov.fj/images/Laws/Press%20Release%20-%20Government%20Approves%202018%20Public%20Holidays.pdf)
         * [2019](https://web.archive.org/web/20191018023027/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
         * [2020](https://web.archive.org/web/20210103183942/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
         * [2021-2022](https://web.archive.org/web/20221223004409/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
         * [2023](https://web.archive.org/web/20231129154609/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
         * [2024](https://web.archive.org/web/20250121185434/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
         * [2025](https://web.archive.org/web/20250318092311/https://www.fiji.gov.fj/About-Fiji/Public-Holidays)
    """

    country = "FJ"
    supported_categories = (PUBLIC, WORKDAY)
    # %s (estimated).
    estimated_label = "%s (estimated)"
    # %s (observed).
    observed_label = "%s (observed)"
    # %s (observed, estimated).
    observed_estimated_label = "%s (observed, estimated)"
    # Act No. 13 of 2015
    start_year = 2016

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=FijiHinduHolidays, show_estimated=True)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=FijiIslamicHolidays, show_estimated=islamic_show_estimated
        )
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Saturday.
        self._add_holy_saturday("Easter Saturday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # National Sports Day.
        if self._year <= 2018:
            self._add_holiday_last_fri_of_jun("National Sports Day")

        if self._year <= 2022:
            # Constitution Day.
            self._add_observed(self._add_holiday_sep_7("Constitution Day"))

        if self._year >= 2023:
            self._move_holiday_forced(
                # Girmit Day.
                self._add_holiday_may_14("Girmit Day"),
                rule=ALL_TO_NEAREST_MON,
            )

        # Ratu Sir Lala Sukuna Day.
        name = "Ratu Sir Lala Sukuna Day"
        if self._year == 2023:
            self._add_holiday_last_mon_of_may(name)
        elif self._year >= 2024:
            self._add_holiday_last_fri_of_may(name)

        # Fiji Day.
        self._add_holiday_oct_10("Fiji Day")

        # Diwali.
        self._add_observed(self._add_diwali_india("Diwali"))

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two("Boxing Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # Prophet Mohammed's Birthday.
        for dt in self._add_mawlid_day("Prophet Mohammed's Birthday"):
            self._add_observed(dt)

    def _populate_workday_holidays(self):
        if self._year >= 2023:
            # Constitution Day.
            self._add_holiday_sep_7("Constitution Day")


class FJ(Fiji):
    pass


class FJI(Fiji):
    pass


class FijiHinduHolidays(_CustomHinduHolidays):
    # https://web.archive.org/web/20250413132828/https://web.archive.org/web/20240724121605/https://www.timeanddate.com/holidays/fiji/diwali
    DIWALI_INDIA_DATES = {
        2016: (OCT, 31),
        2017: (OCT, 19),
        2018: (NOV, 7),
        2019: (OCT, 28),
        2020: (NOV, 14),
        2021: (NOV, 4),
        2022: (OCT, 25),
        2023: (NOV, 13),
        2024: (NOV, 1),
        2025: (OCT, 21),
    }


class FijiIslamicHolidays(_CustomIslamicHolidays):
    MAWLID_DATES_CONFIRMED_YEARS = (2016, 2025)
    MAWLID_DATES = {
        2016: (DEC, 12),
        2017: (DEC, 2),
        2018: (NOV, 19),
        2020: (OCT, 31),
        2022: (OCT, 7),  # looks like observed on FRI
        2023: (SEP, 30),
        2024: (SEP, 16),
        2025: (SEP, 6),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Honduras(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Honduras holidays.

    References:
        * [Art. 339 of Labor Code](https://web.archive.org/web/20240201202118/http://ilo.org/dyn/natlex/docs/WEBTEXT/29076/64849/S59HND01.htm)
        * [Decree 78-2015](https://web.archive.org/web/20250414065149/https://www.tsc.gob.hn/web/leyes/Decreto_78-2015_Traslado_de_Feriados_Octubre.pdf)
    """

    country = "HN"
    default_language = "es"
    supported_languages = ("en_US", "es", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jueves Santo"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Holy Saturday.
        self._add_holy_saturday(tr("Sábado de Gloria"))

        # Panamerican Day.
        self._add_holiday_apr_14(tr("Día de las Américas"))

        # Labor Day.
        self._add_labor_day(tr("Día del Trabajo"))

        # Independence Day.
        self._add_holiday_sep_15(tr("Día de la Independencia"))

        # Decree 78-2015.
        if self._year <= 2014:
            # Morazan's Day.
            self._add_holiday_oct_3(tr("Día de Morazán"))

            # Columbus Day.
            self._add_columbus_day(tr("Día de la Raza"))

            # Army Day.
            self._add_holiday_oct_21(tr("Día de las Fuerzas Armadas"))
        else:
            # Morazan Weekend.
            name = tr("Semana Morazánica")
            # First Wednesday of October from 12 noon to Saturday 12 noon.
            self._add_holiday_1st_wed_of_oct(name)
            self._add_holiday_1_day_past_1st_wed_of_oct(name)
            self._add_holiday_2_days_past_1st_wed_of_oct(name)

        # Christmas Day.
        self._add_christmas_day(tr("Navidad"))


class HN(Honduras):
    pass


class HND(Honduras):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class GuineaBissau(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Guinea-Bissau holidays.

    References:
        * [Ley núm. 7/2022, de 18 de julio, que aprueba el Código del Trabajo](https://archive.org/details/742984686-co-digo-de-trabalho-lei-no-7-2022)
    """

    country = "GW"
    default_language = "pt_GW"
    # %s (estimated).
    estimated_label = tr("%s (prevista)")
    supported_languages = ("en_US", "pt_GW")
    start_year = 2023

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=GuineaBissauIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Ano Novo"))

        # National Heroes' Day.
        self._add_holiday_jan_20(tr("Dia dos Heróis Nacionais"))

        # Day of the Beginning of the Armed Struggle.
        self._add_holiday_jan_23(tr("Dia do Início da Luta Armada"))

        # International Women's Day.
        self._add_womens_day(tr("Dia Internacional da Mulher"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Páscoa"))

        # Worker's Day.
        self._add_labor_day(tr("Dia do Trabalhador"))

        # Pidjiguiti Day.
        self._add_holiday_aug_3(tr("Dia de Pidjiguiti"))

        # Independence Day.
        self._add_holiday_sep_24(tr("Dia da Independência"))

        # Christmas Day.
        self._add_christmas_day(tr("Dia de Natal"))

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("Korité"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("Tabaski"))


class GW(GuineaBissau):
    pass


class GNB(GuineaBissau):
    pass


class GuineaBissauIslamicHolidays(_CustomIslamicHolidays):
    # http://web.archive.org/web/20250811201914/https://www.timeanddate.com/holidays/guinea-bissau/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2023, 2025)

    # http://web.archive.org/web/20250811202225/https://www.timeanddate.com/holidays/guinea-bissau/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2023, 2025)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

import re
import uuid
from datetime import date, datetime, timezone
from pathlib import Path
from typing import TYPE_CHECKING

from holidays.calendars.gregorian import _timedelta
from holidays.version import __version__

if TYPE_CHECKING:
    from collections.abc import Iterable

    from holidays.holiday_base import HolidayBase

# iCal-specific constants
CONTENT_LINE_MAX_LENGTH = 75
CONTENT_LINE_DELIMITER = "\r\n"
CONTENT_LINE_DELIMITER_WRAP = f"{CONTENT_LINE_DELIMITER} "


class ICalExporter:
    def __init__(self, instance: HolidayBase, show_language: bool = False) -> None:
        """Initialize iCalendar exporter.

        Args:
            show_language:
                Determines whether to include the `;LANGUAGE=` attribute in the
                `SUMMARY` field. Defaults to `False`.

                If the [`HolidayBase`][holidays.holiday_base.HolidayBase] object
                has a `language` attribute, it will be used. Otherwise,
                `default_language` will be used if available.

                If neither attribute exists and `show_language=True`, an
                exception will be raised.

            instance:
                [`HolidayBase`][holidays.holiday_base.HolidayBase] object
                containing holiday data.
        """
        self.holidays = instance
        self.show_language = show_language
        self.ical_timestamp = datetime.now(timezone.utc).strftime("%Y%m%dT%H%M%SZ")
        self.holidays_version = __version__
        language = getattr(self.holidays, "language", None) or getattr(
            self.holidays, "default_language", None
        )
        self.language = (
            self._validate_language(language)
            if isinstance(language, str)
            and language in getattr(self.holidays, "supported_languages", [])
            else None
        )

        if self.show_language and self.language is None:
            raise ValueError("LANGUAGE cannot be included because the language code is missing.")

    def _validate_language(self, language: str) -> str:
        """Validate the language code to ensure it complies with
        [RFC 5646](https://datatracker.ietf.org/doc/html/rfc5646).

        In the current implementation, all languages must comply with
        either [ISO 639-1 or ISO 639-2](https://www.loc.gov/standards/iso639-2/php/code_list.php)
        if specified (part of [RFC 5646](https://datatracker.ietf.org/doc/html/rfc5646)).

        Args:
            language:
                The language code to validate.

        Returns:
            Validated language code.
        """
        # Remove whitespace (if any), transforms HolidaysBase default to RFC 5646 compliant
        # i.e. `en_US` to `en-US`.
        language = language.strip().replace("_", "-")

        # ISO 639-1 and ISO 639-2 patterns, in compliance with RFC 5646.
        iso639_pattern = re.compile(r"^[a-z]{2,3}(?:-[A-Z]{2})?$")

        if not iso639_pattern.fullmatch(language):
            raise ValueError(
                f"Invalid language tag: '{language}'. Expected format follows "
                "ISO 639-1 or ISO 639-2, e.g., 'en', 'en-US'. For more details, "
                "refer to: https://www.loc.gov/standards/iso639-2/php/code_list.php."
            )
        return language

    def _fold_line(self, line: str) -> str:
        """Fold long lines according to
        [RFC 5545](https://datatracker.ietf.org/doc/html/rfc5545).

        Content lines SHOULD NOT exceed 75 octets. If a line is too long,
        it must be split into multiple lines, with each continuation line
        starting with a space.

        Args:
            line:
                The content line to be folded.

        Returns:
            The folded content line.
        """
        if line.isascii():
            # Simple split for ASCII: every (CONTENT_LINE_MAX_LENGTH - 1) chars,
            # as first char of the next line is space
            if len(line) > CONTENT_LINE_MAX_LENGTH:
                return CONTENT_LINE_DELIMITER_WRAP.join(
                    line[i : i + CONTENT_LINE_MAX_LENGTH - 1]
                    for i in range(0, len(line), CONTENT_LINE_MAX_LENGTH - 1)
                )

        elif len(line.encode()) > CONTENT_LINE_MAX_LENGTH:
            # Handle non-ASCII text while respecting byte length
            parts = []
            part_start = 0
            part_len = 0
            for i, char in enumerate(line):
                char_byte_len = len(char.encode())
                part_len += char_byte_len

                if part_len > CONTENT_LINE_MAX_LENGTH:
                    parts.append(line[part_start:i])
                    part_start = i
                    part_len = char_byte_len + 1  # line start with space

            parts.append(line[part_start:])
            return CONTENT_LINE_DELIMITER_WRAP.join(parts)

        # Return as-is if it doesn't exceed the limit
        return line

    def _generate_event(
        self, dt: date, holiday_name: str, holiday_length: int = 1
    ) -> Iterable[str]:
        """Generate a single holiday event.

        Args:
            dt:
                Holiday date.

            holiday_name:
                Holiday name.

            holiday_length:
                Holiday length in days, default to 1.

        Returns:
            Iterable of iCalendar format event lines.
        """
        # Escape special characters per RFC 5545.
        # SEMICOLON is used as a delimiter in HolidayBase (HOLIDAY_NAME_DELIMITER = "; "),
        # so a name with a semicolon gets split into two separate `VEVENT`s.
        sanitized_holiday_name = (
            holiday_name.replace("\\", "\\\\").replace(",", "\\,").replace(":", "\\:")
        )
        event_uid = f"{uuid.uuid4()}@{self.holidays_version}.holidays.local"
        language_tag = f";LANGUAGE={self.language}" if self.show_language else ""

        yield "BEGIN:VEVENT"
        yield f"DTSTAMP:{self.ical_timestamp}"
        yield f"UID:{event_uid}"
        yield self._fold_line(f"SUMMARY{language_tag}:{sanitized_holiday_name}")
        yield f"DTSTART;VALUE=DATE:{dt:%Y%m%d}"
        yield f"DURATION:P{holiday_length}D"
        if len(self.holidays.categories) == 1:
            yield f"CATEGORIES:{next(iter(self.holidays.categories)).upper()}"
        yield "END:VEVENT"

    def generate(self, return_bytes: bool = False) -> str | bytes:
        """Generate iCalendar data.

        Args:
            return_bytes:
                If `True`, return bytes instead of string.

        Returns:
            The complete iCalendar data
            (string or UTF-8 bytes depending on return_bytes).
        """
        lines = [
            "BEGIN:VCALENDAR",
            f"PRODID:-//Vacanza//Open World Holidays Framework v{self.holidays_version}//EN",
            "VERSION:2.0",
            "CALSCALE:GREGORIAN",
        ]

        sorted_dates = sorted(self.holidays.keys())
        # Merged continuous holiday with the same name and use `DURATION` instead.
        n = len(sorted_dates)
        i = 0
        while i < n:
            dt = sorted_dates[i]
            names = self.holidays.get_list(dt)

            for name in names:
                days = 1
                while (
                    i + days < n
                    and sorted_dates[i + days] == _timedelta(sorted_dates[i], days)
                    and name in self.holidays.get_list(sorted_dates[i + days])
                ):
                    days += 1

                lines.extend(self._generate_event(dt, name, days))

            i += days

        lines.append("END:VCALENDAR")
        lines.append("")

        output = CONTENT_LINE_DELIMITER.join(lines)
        return output.encode() if return_bytes else output

    def save_ics(self, file_path: str | Path) -> None:
        """Export the calendar data to a `.ics` file.

        While [RFC 5545](https://datatracker.ietf.org/doc/html/rfc5545) does not explicitly
        restrict filenames for `.ics` files, it is still advisable to follow general filesystem
        conventions and avoid problematic characters.

        Args:
            file_path:
                Path to save the `.ics` file, including the filename (with extension).
        """
        # Generate and write out content (always in bytes for .ics)
        content = self.generate(return_bytes=True)
        if not content:
            raise ValueError("Generated content is empty or invalid.")

        Path(file_path).write_bytes(content)  # type: ignore[arg-type]



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, MAR, SEP, NOV
from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, TUE_WED_THU_TO_NEXT_FRI


class Brazil(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Brazil holidays.

    References:
        * <https://pt.wikipedia.org/wiki/Feriados_no_Brasil>
        * [Decreto n. 155-B, de 14.01.1890](https://web.archive.org/web/20241226133739/https://www2.camara.leg.br/legin/fed/decret/1824-1899/decreto-155-b-14-janeiro-1890-517534-publicacaooriginal-1-pe.html)
        * [Decreto n. 19.488, de 15.12.1930](https://web.archive.org/web/20241006041503/http://camara.leg.br/legin/fed/decret/1930-1939/decreto-19488-15-dezembro-1930-508040-republicacao-85201-pe.html)
        * [Lei n. 662, de 6.04.1949](https://web.archive.org/web/20240913060643/https://www2.camara.leg.br/legin/fed/lei/1940-1949/lei-662-6-abril-1949-347136-publicacaooriginal-1-pl.html)
        * [Lei n. 14.759, de 21.12.2023](https://web.archive.org/web/20250402234552/https://www2.camara.leg.br/legin/fed/lei/2023/lei-14759-21-dezembro-2023-795091-publicacaooriginal-170522-pl.html)

    Subdivisions Holidays References:
        * Parana:
            * [Lei n. 18.384, de 17.12.2014](https://web.archive.org/web/20240613145927/https://www.legislacao.pr.gov.br/legislacao/pesquisarAto.do?action=exibir&codAto=134573&codItemAto=816343)
        * Pernambuco:
            * [Lei n. 13.386, de 24.12.2007](https://web.archive.org/web/20210727183402/http://legis.alepe.pe.gov.br/texto.aspx?id=3767&tipo=TEXTOORIGINAL)
            * [Lei n. 16.059, de 08.06.2017](https://web.archive.org/web/20250420091334/https://legis.alepe.pe.gov.br/texto.aspx?id=28224&tipo=TEXTOORIGINAL)
            * [Pontos Facultativos, Feriados, Festas](https://web.archive.org/web/20200624203011/https://www.pe.gov.br/conheca/datas/)
        * Rio de Janeiro:
            * [Lei n. 5198, de 05.03.2008](https://web.archive.org/web/20160303175842/http://alerjln1.alerj.rj.gov.br/CONTLEI.NSF/c8aa0900025feef6032564ec0060dfff/1baf90ca125ff96f8325740a00776600)
            * [Lei n. 5243, de 14.05.2008](https://web.archive.org/web/20201106190517/http://alerjln1.alerj.rj.gov.br/contlei.nsf/351902c88b9f0cde03256bcf00607a5b/063f7c027766eab48325744a007a4ab0)
        * São Paulo Capital:
            * [Lei Municipal n. 13.707, de 07.01.2004](https://web.archive.org/web/20250519045554/http://legislacao.prefeitura.sp.gov.br/leis/lei-13707-de-7-de-janeiro-de-2004/consolidado)
            * [Anexo Único Integrante do Decreto n. 56.756, de 04.01.2016](https://web.archive.org/web/20251224031221/https://legislacao.prefeitura.sp.gov.br/leis/decreto-56756-de-04-de-janeiro-de-2016/anexo/5ec3d7a71411926001a56c56/Anexo%20Único%20do%20Decreto%20nº%2056.756_2016.pdf)
    """

    country = "BR"
    default_language = "pt_BR"
    # Decreto n. 155-B, de 14.01.1890
    start_year = 1890
    subdivisions = (
        "AC",
        "AL",
        "AM",
        "AP",
        "BA",
        "CE",
        "DF",
        "ES",
        "GO",
        "MA",
        "MG",
        "MS",
        "MT",
        "PA",
        "PB",
        "PE",
        "PI",
        "PR",
        "RJ",
        "RN",
        "RO",
        "RR",
        "RS",
        "SC",
        "SE",
        "SP",
        "TO",
        # Cities.
        "São Paulo Capital",
    )
    subdivisions_aliases = {
        "Acre": "AC",
        "Alagoas": "AL",
        "Amazonas": "AM",
        "Amapá": "AP",
        "Bahia": "BA",
        "Ceará": "CE",
        "Distrito Federal": "DF",
        "Espírito Santo": "ES",
        "Goiás": "GO",
        "Maranhão": "MA",
        "Minas Gerais": "MG",
        "Mato Grosso do Sul": "MS",
        "Mato Grosso": "MT",
        "Pará": "PA",
        "Paraíba": "PB",
        "Pernambuco": "PE",
        "Piauí": "PI",
        "Paraná": "PR",
        "Rio de Janeiro": "RJ",
        "Rio Grande do Norte": "RN",
        "Rondônia": "RO",
        "Roraima": "RR",
        "Rio Grande do Sul": "RS",
        "Santa Catarina": "SC",
        "Sergipe": "SE",
        "São Paulo": "SP",
        "Tocantins": "TO",
    }
    supported_categories = (OPTIONAL, PUBLIC)
    supported_languages = ("en_US", "pt_BR", "uk")

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Universal Fraternization Day.
        self._add_new_years_day(tr("Confraternização Universal"))

        if 1892 <= self._year <= 1930:
            # Republic Constitution Day.
            self._add_holiday_feb_24(tr("Constituição da República"))

        # Good Friday.
        self._add_good_friday(tr("Sexta-feira Santa"))

        if self._year not in {1931, 1932}:
            # Tiradentes' Day.
            self._add_holiday_apr_21(tr("Tiradentes"))

        if self._year >= 1925:
            # Worker's Day.
            self._add_labor_day(tr("Dia do Trabalhador"))

        if self._year <= 1930 or 1936 <= self._year <= 1948:
            # Discovery of Brazil.
            self._add_holiday_may_3(tr("Descobrimento do Brasil"))

        if self._year <= 1930:
            # Abolition of slavery in Brazil.
            self._add_holiday_may_13(tr("Abolição da escravidão no Brasil"))

            # Freedom and Independence of American Peoples.
            self._add_holiday_jul_14(tr("Liberdade e Independência dos Povos Americanos"))

        # Independence Day.
        self._add_holiday_sep_7(tr("Independência do Brasil"))

        if self._year <= 1930 or 1936 <= self._year <= 1948:
            # Discovery of America.
            self._add_columbus_day(tr("Descobrimento da América"))

        if self._year >= 1980:
            # Our Lady of Aparecida.
            self._add_holiday_oct_12(tr("Nossa Senhora Aparecida"))

        # All Souls' Day.
        self._add_all_souls_day(tr("Finados"))

        # Republic Proclamation Day.
        self._add_holiday_nov_15(tr("Proclamação da República"))

        if self._year >= 2024:
            # National Day of Zumbi and Black Awareness.
            self._add_holiday_nov_20(tr("Dia Nacional de Zumbi e da Consciência Negra"))

        if self._year >= 1922:
            # Christmas Day.
            self._add_christmas_day(tr("Natal"))

        if self.subdiv == "São Paulo Capital":
            self._populate_subdiv_sao_paulo_capital_public_holidays()

    def _populate_optional_holidays(self):
        # Carnival.
        name = tr("Carnaval")
        self._add_carnival_monday(name)
        self._add_carnival_tuesday(name)

        # Ash Wednesday.
        self._add_ash_wednesday(tr("Início da Quaresma"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Corpus Christi"))

        # Public Servant's Day.
        self._add_holiday_oct_28(tr("Dia do Servidor Público"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Véspera de Natal"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Véspera de Ano-Novo"))

    def _populate_subdiv_holidays(self):
        # Lei n. 9.093, de 12.09.1995
        # Lei Municipal n. 7.008, de 6.04.1967
        if self._year >= 1996 or self.subdiv == "São Paulo Capital":
            super()._populate_subdiv_holidays()

    def _populate_subdiv_ac_public_holidays(self):
        def get_movable_acre(*args) -> date:
            dt = date(self._year, *args)
            return (
                dt_observed
                if self._year >= 2009
                and (dt_observed := self._get_observed_date(dt, TUE_WED_THU_TO_NEXT_FRI))
                else dt
            )

        if self._year >= 2005:
            # Evangelical Day.
            self._add_holiday(tr("Dia do Evangélico"), get_movable_acre(JAN, 23))

        if self._year >= 2002:
            # International Women's Day.
            self._add_holiday(tr("Dia Internacional da Mulher"), get_movable_acre(MAR, 8))

        # Founding of Acre.
        self._add_holiday_jun_15(tr("Aniversário do Acre"))

        if self._year >= 2004:
            # Amazonia Day.
            self._add_holiday(tr("Dia da Amazônia"), get_movable_acre(SEP, 5))

        # Signing of the Petropolis Treaty.
        self._add_holiday(tr("Assinatura do Tratado de Petrópolis"), get_movable_acre(NOV, 17))

    def _populate_subdiv_al_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("São João"))

        # Saint Peter's Day.
        self._add_saints_peter_and_paul_day(tr("São Pedro"))

        # Political Emancipation of Alagoas.
        self._add_holiday_sep_16(tr("Emancipação Política de Alagoas"))

        if self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

        if self._year >= 2013:
            # Evangelical Day.
            self._add_holiday_nov_30(tr("Dia do Evangélico"))

    def _populate_subdiv_am_public_holidays(self):
        # Elevation of Amazonas to province.
        self._add_holiday_sep_5(tr("Elevação do Amazonas à categoria de província"))

        if 2010 <= self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

    def _populate_subdiv_ap_public_holidays(self):
        if self._year >= 2003:
            # Saint Joseph's Day.
            self._add_saint_josephs_day(tr("São José"))

        if self._year >= 2012:
            # Saint James' Day.
            self._add_saint_james_day(tr("São Tiago"))

        # Creation of the Federal Territory.
        self._add_holiday_sep_13(tr("Criação do Território Federal"))

        if 2008 <= self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

    def _populate_subdiv_ba_public_holidays(self):
        # Bahia Independence Day.
        self._add_holiday_jul_2(tr("Independência da Bahia"))

    def _populate_subdiv_ce_public_holidays(self):
        self._add_saint_josephs_day(tr("São José"))

        # Abolition of slavery in Ceará.
        self._add_holiday_mar_25(tr("Abolição da escravidão no Ceará"))

        if self._year >= 2004:
            # Our Lady of Assumption.
            self._add_assumption_of_mary_day(tr("Nossa Senhora da Assunção"))

    def _populate_subdiv_df_public_holidays(self):
        # Founding of Brasilia.
        self._add_holiday_apr_21(tr("Fundação de Brasília"))

        # Evangelical Day.
        self._add_holiday_nov_30(tr("Dia do Evangélico"))

    def _populate_subdiv_es_public_holidays(self):
        if self._year >= 2020:
            # Our Lady of Penha.
            self._add_holiday_8_days_past_easter(tr("Nossa Senhora da Penha"))

    def _populate_subdiv_go_public_holidays(self):
        # Foundation of Goiás city.
        self._add_holiday_jul_26(tr("Fundação da cidade de Goiás"))

        # Foundation of Goiânia.
        self._add_holiday_oct_24(tr("Pedra fundamental de Goiânia"))

    def _populate_subdiv_ma_public_holidays(self):
        # Maranhão joining to independence of Brazil.
        self._add_holiday_jul_28(tr("Adesão do Maranhão à independência do Brasil"))

    def _populate_subdiv_mg_public_holidays(self):
        # Tiradentes' Execution.
        self._add_holiday_apr_21(tr("Execução de Tiradentes"))

    def _populate_subdiv_ms_public_holidays(self):
        # State Creation Day.
        self._add_holiday_oct_11(tr("Criação do Estado"))

    def _populate_subdiv_mt_public_holidays(self):
        if 2003 <= self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

    def _populate_subdiv_pa_public_holidays(self):
        # Grão-Pará joining to independence of Brazil.
        self._add_holiday_aug_15(tr("Adesão do Grão-Pará à independência do Brasil"))

    def _populate_subdiv_pb_public_holidays(self):
        # State Founding Day.
        self._add_holiday_aug_5(tr("Fundação do Estado"))

    def _populate_subdiv_pe_public_holidays(self):
        # Established by Lei n. 13.386.
        if self._year >= 2008:
            # Pernambuco Revolution of 1817.
            name = tr("Revolução Pernambucana de 1817")
            # Made fixed by Lei n. 16.059.
            if self._year >= 2018:
                self._add_holiday_mar_6(name)
            else:
                self._add_holiday_1st_sun_of_mar(name)

    def _populate_subdiv_pi_public_holidays(self):
        # Piauí Day.
        self._add_holiday_oct_19(tr("Dia do Piauí"))

    def _populate_subdiv_pr_public_holidays(self):
        # Established by Lei n. 18.397.
        if self._year >= 2014:
            # Our Lady of Rocio.
            self._add_holiday_nov_15(tr("Nossa Senhora do Rocio"))
        # Abolished by Lei n. 18.384.
        else:
            # Political Emancipation of Paraná.
            self._add_holiday_dec_19(tr("Emancipação do Paraná"))

    def _populate_subdiv_rj_public_holidays(self):
        # Established by Lei n. 5243.
        if self._year >= 2009:
            # Carnival.
            self._add_carnival_tuesday(tr("Carnaval"))

        # Established by Lei n. 5198.
        if self._year >= 2008:
            # Saint George's Day.
            self._add_saint_georges_day(tr("São Jorge"))

        if 2002 <= self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

    def _populate_subdiv_rn_public_holidays(self):
        if self._year >= 2000:
            # Rio Grande do Norte Day.
            self._add_holiday_aug_7(tr("Dia do Rio Grande do Norte"))

        if self._year >= 2007:
            # Uruaçu and Cunhaú Martyrs Day.
            self._add_holiday_oct_3(tr("Mártires de Cunhaú e Uruaçu"))

    def _populate_subdiv_ro_public_holidays(self):
        # State Creation Day.
        self._add_holiday_jan_4(tr("Criação do Estado"))

        if self._year >= 2002:
            # Evangelical Day.
            self._add_holiday_jun_18(tr("Dia do Evangélico"))

    def _populate_subdiv_rr_public_holidays(self):
        # State Creation Day.
        self._add_holiday_oct_5(tr("Criação do Estado"))

    def _populate_subdiv_rs_public_holidays(self):
        # Gaucho Day.
        self._add_holiday_sep_20(tr("Dia do Gaúcho"))

    def _populate_subdiv_sc_public_holidays(self):
        if self._year >= 2004:
            # Santa Catarina State Day.
            name = tr("Dia do Estado de Santa Catarina")
            if self._year >= 2005:
                self._add_holiday_1st_sun_from_aug_11(name)
            else:
                self._add_holiday_aug_11(name)

        # Saint Catherine of Alexandria Day.
        name = tr("Dia de Santa Catarina de Alexandria")
        if 1999 <= self._year != 2004:
            self._add_holiday_1st_sun_from_nov_25(name)
        else:
            self._add_holiday_nov_25(name)

    def _populate_subdiv_se_public_holidays(self):
        # Sergipe Political Emancipation Day.
        self._add_holiday_jul_8(tr("Emancipação política de Sergipe"))

    def _populate_subdiv_sp_public_holidays(self):
        if self._year >= 1997:
            # Constitutionalist Revolution.
            self._add_holiday_jul_9(tr("Revolução Constitucionalista"))

    def _populate_subdiv_sao_paulo_capital_public_holidays(self):
        self._populate_subdiv_sp_public_holidays()

        # Lei Municipal n. 7.008, de 06.04.1967.
        if self._year >= 1968:
            # São Paulo City Anniversary.
            self._add_holiday_jan_25(tr("Aniversário da Cidade de São Paulo"))

            # Corpus Christi.
            self._add_corpus_christi_day(tr("Corpus Christi"))

        # Established by Lei Municipal n. 13.707, de 07.01.2004.
        if 2004 <= self._year <= 2023:
            # Black Awareness Day.
            self._add_holiday_nov_20(tr("Consciência Negra"))

    def _populate_subdiv_to_public_holidays(self):
        if self._year >= 1998:
            # Autonomy Day.
            self._add_holiday_mar_18(tr("Dia da Autonomia"))

        # Our Lady of Nativity.
        self._add_nativity_of_mary_day(tr("Nossa Senhora da Natividade"))

        # State Creation Day.
        self._add_holiday_oct_5(tr("Criação do Estado"))


class BR(Brazil):
    pass


class BRA(Brazil):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Armenia(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Armenia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Armenia>
        * [Law HO-37 of Mar 16, 1996](https://web.archive.org/web/20260122214757/https://www.arlis.am/hy/acts/259)
        * [Law HO-117 of Dec 22, 2000](https://web.archive.org/web/20250717094812/https://www.arlis.am/hy/acts/801)
        * [Law HO-118 of Dec 22, 2000](https://web.archive.org/web/20250717090032/https://www.arlis.am/hy/acts/802)
        * [Law HO-200 of Jun 24, 2001](https://web.archive.org/web/20250427132425/http://www.parliament.am/legislation.php?sel=show&ID=1274&lang=arm)
        * [Law HO-228 of Oct 11, 2001](https://web.archive.org/web/20250427175053/http://www.parliament.am/legislation.php?ID=1622&lang=arm&sel=show)
        * [Law HO-335 of Jun 04, 2002](https://web.archive.org/web/20251210090018/http://www.parliament.am/legislation.php?sel=show&ID=1623&lang=arm)
        * [Law HO-466 of Dec 19, 2002](https://web.archive.org/web/20251209213043/http://www.parliament.am/legislation.php?sel=show&ID=1624&lang=arm)
        * [Law HO-80 of May 2, 2005](https://web.archive.org/web/20241211151029/http://www.parliament.am/legislation.php?sel=show&ID=2284&lang=arm)
        * [Law HO-105 of Jun 16, 2005](https://web.archive.org/web/20250508000307/http://www.parliament.am/legislation.php?lang=arm&sel=show&ID=2326)
        * [Law HO-100 of Jun 21, 2006](https://web.archive.org/web/20250428015807/http://www.parliament.am/legislation.php?ID=2648&lang=arm&sel=show)
        * [Law HO-221 of Dec 16, 2009](https://web.archive.org/web/20250501031654/http://www.parliament.am/legislation.php?ID=3737&lang=arm&sel=show)
        * [Law HO-134 of Oct 22, 2010](https://web.archive.org/web/20231201154954/http://www.parliament.am/legislation.php?sel=show&ID=3886&lang=arm)
        * [Law HO-127 of May 6, 2011](https://web.archive.org/web/20231201160900/http://www.parliament.am/legislation.php?sel=show&ID=4177&lang=arm)
        * [Law HO-142 of May 21, 2011](https://web.archive.org/web/20251210013331/http://www.parliament.am/legislation.php?sel=show&ID=4198&lang=arm)
        * [Law HO-15 of Apr 15, 2015](https://web.archive.org/web/20250510074755/http://parliament.am/legislation.php?ID=5221&lang=arm&sel=show)
        * [Law HO-187 of Dec 30, 2015](https://web.archive.org/web/20250510003259/http://parliament.am/legislation.php?sel=show&ID=5390&lang=arm)
        * [Law HO-67 of Mar 23, 2017](https://web.archive.org/web/20250428212817/http://www.parliament.am/legislation.php?sel=show&ID=5795&lang=arm)
        * [Law HO-14-N of Apr 19, 2019](https://web.archive.org/web/20231201160233/http://www.parliament.am/legislation.php?sel=show&ID=6551&lang=arm)
        * [Law HO-83-N of Jul 3, 2019](https://web.archive.org/web/20231201160446/http://www.parliament.am/legislation.php?sel=show&ID=6640&lang=arm)
        * [Law HO-191-N of Nov 13, 2019](https://web.archive.org/web/20231201163636/http://www.parliament.am/legislation.php?sel=show&ID=6751&lang=arm)
        * [Law HO-82-N of Mar 15, 2021](https://web.archive.org/web/20231201153032/http://www.parliament.am/legislation.php?sel=show&ID=7506&lang=arm)
        * [Law HO-191-N of May 18, 2021](https://web.archive.org/web/20250505174605/http://www.parliament.am/legislation.php?lang=arm&sel=show&ID=7622)
        * [Law HO-362-N of Nov 29, 2021](https://web.archive.org/web/20250427183056/http://www.parliament.am/legislation.php?sel=show&ID=7828&lang=arm)
        * [Law HO-147-N of May 2, 2024](https://web.archive.org/web/20250427204734/http://www.parliament.am/legislation.php?ID=9191&lang=arm&sel=show)
        * [Law HO-370-N of Nov 7, 2024](https://web.archive.org/web/20250505152132/http://www.parliament.am/legislation.php?sel=show&ID=9424&lang=arm)
        * [Law HO-1-N of Jan 20, 2026](https://web.archive.org/web/20260122152327/http://www.parliament.am/legislation.php?sel=show&ID=10146&lang=arm)
    """

    country = "AM"
    default_language = "hy"
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "hy")
    start_year = 1991
    subdivisions = (
        "AG",  # Aragac̣otn.
        "AR",  # Ararat.
        "AV",  # Armavir.
        "ER",  # Erevan.
        "GR",  # Geġark'unik'.
        "KT",  # Kotayk'.
        "LO",  # Loṙi.
        "SH",  # Širak.
        "SU",  # Syunik'.
        "TV",  # Tavuš.
        "VD",  # Vayoć Jor.
    )
    subdivisions_aliases = {
        "Aragac̣otn": "AG",
        "Aragatsotn": "AG",
        "Ararat": "AR",
        "Armavir": "AV",
        "Erevan": "ER",
        "Geġark'unik'": "GR",
        "Gegharkunik": "GR",
        "Kotayk'": "KT",
        "Kotayk": "KT",
        "Loṙi": "LO",
        "Lori": "LO",
        "Širak": "SH",
        "Shirak": "SH",
        "Syunik'": "SU",
        "Syunik": "SU",
        "Tavuš": "TV",
        "Tavush": "TV",
        "Vayoć Jor": "VD",
        "Vayots Dzor": "VD",
    }

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, ArmeniaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Նոր տարվա օր")
        self._add_new_years_day(name)
        self._add_new_years_day_two(name)

        # Christmas and Epiphany.
        self._add_holiday_jan_6(tr("Սուրբ Ծնունդ եւ Հայտնություն"))

        # Established by Law HO-221 of Dec 16, 2009.
        # Abolished by Law HO-362-N of Nov 29, 2021.
        if 2010 <= self._year <= 2021:
            # Christmas Holidays.
            name = tr("նախածննդյան տոներ")
            self._add_holiday_jan_3(name)
            self._add_holiday_jan_4(name)
            self._add_holiday_jan_5(name)

            # Day of Remembrance of the Dead.
            self._add_holiday_jan_7(tr("Մեռելոց հիշատակի օր"))

        # Established by Law HO-1-N of Jan 20, 2026.
        if self._year >= 2026:
            # Day of Remembrance for the Fallen Defenders of the Fatherland.
            self._add_holiday_jan_27(tr("Հայրենիքի պաշտպանության համար զոհվածների հիշատակի օր"))

        # Made non-working by Law HO-466 of Dec 19, 2002.
        if self._year >= 2003:
            # Army Day.
            self._add_holiday_jan_28(tr("Բանակի օր"))

        # Established by Law HO-118 of Dec 22, 2000.
        # Renamed by Law HO-200 of Jun 24, 2001.
        if self._year >= 2001:
            self._add_womens_day(
                # Women's Day.
                tr("Կանանց տոն")
                if self._year >= 2002
                # International Women's Day.
                else tr("Կանանց միջազգային օր")
            )

        # Made working by Law HO-200 of Jun 24, 2001.
        if 1994 <= self._year <= 2001:
            # Motherhood and Beauty Day.
            self._add_holiday_apr_7(tr("Մայրության և գեղեցկության տոն"))

        # Renamed by Law HO-15 of Apr 15, 2015.
        self._add_holiday_apr_24(
            # Day of Remembrance of the Victims of Armenian Genocide.
            tr("Հայոց ցեղասպանության զոհերի հիշատակի օր")
            if self._year >= 2015
            # Day of Remembrance of the Victims of Genocide.
            else tr("Ցեղասպանության զոհերի հիշատակի օր")
        )

        # Established by Law HO-117 of Dec 22, 2000.
        # Renamed by Law HO-200 of Jun 24, 2001.
        if self._year >= 2001:
            self._add_labor_day(
                # Labor Day.
                tr("Աշխատանքի օր")
                if self._year >= 2002
                # International Day of Workers' Solidarity.
                else tr("Աշխատավորների համերաշխության միջազգային օր")
            )

        if self._year >= 1995:
            self._add_world_war_two_victory_day(
                # Victory and Peace Day.
                tr("Հաղթանակի և Խաղաղության տոն"),
                is_western=False,
            )

        # Republic Day.
        self._add_holiday_may_28(tr("Հանրապետության օր"))

        if self._year >= 1996:
            # Constitution Day.
            self._add_holiday_jul_5(tr("Սահմանադրության օր"))

        if self._year >= 1992:
            # Independence Day.
            self._add_holiday_sep_21(tr("Անկախության օր"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Նոր տարվա գիշեր"))

    def _populate_workday_holidays(self):
        # Established by Law HO-200 of Jun 24, 2001.
        # Made non-working by Law HO-466 of Dec 19, 2002.
        if self._year == 2002:
            # Army Day.
            self._add_holiday_jan_28(tr("Բանակի օր"))

        # Established by Law HO-80 of May 2, 2005.
        if self._year >= 2006:
            # Mother Language Day.
            self._add_holiday_feb_21(tr("Մայրենի լեզվի օր"))

        # Established by Law HO-200 of Jun 24, 2001.
        if self._year >= 2002:
            # Saint Vardanants' Day.
            self._add_holiday_52_days_prior_easter(tr("Սուրբ Վարդանանց տոն"))

        # Established by Law HO-105 of Jun 16, 2005.
        if self._year >= 2006:
            self._add_holiday_feb_28(
                # Day of Remembrance of the Victims of the Massacres Organized in the Azerbaijan
                # SSR and the Protection of the Rights of the Deported Armenian Population.
                tr(
                    "Ադրբեջանական ԽՍՀ-ում կազմակերպված ջարդերի զոհերի հիշատակի եւ "
                    "բռնագաղթված հայ բնակչության իրավունքների պաշտպանության օր"
                )
            )

        # Made working by Law HO-200 of Jun 24, 2001.
        # Renamed by Law HO-335 of Jun 04, 2002.
        if self._year >= 2002:
            self._add_holiday_apr_7(
                # Motherhood and Beauty Day.
                tr("Մայրության և գեղեցկության տոն")
                if self._year >= 2003
                # Motherhood, Beauty and Love Day.
                else tr("Մայրության, գեղեցկության եւ սիրո տոն")
            )

        # Established by Law HO-83-N of Jul 3, 2019.
        if self._year >= 2020:
            # Armenian Cinema Day.
            self._add_holiday_apr_16(tr("Հայ կինոյի օր"))

        # Established by Law HO-191-N of May 18, 2021.
        if self._year >= 2022:
            # Taxpayer Day.
            self._add_holiday_apr_18(tr("Հարկ վճարողի օր"))

        # Established by Law HO-14-N of Apr 19, 2019.
        if self._year >= 2019:
            # Citizen of the Republic of Armenia Day.
            name = tr("Հայաստանի Հանրապետության քաղաքացու օր")
            if self._is_saturday(MAY, 1):
                self._add_holiday_last_sun_of_apr(name)
            else:
                self._add_holiday_last_sat_of_apr(name)

        # Established by Law HO-200 of Jun 24, 2001.
        if self._year >= 2002:
            # Defender's Day.
            self._add_holiday_may_8(tr("Երկրապահի օր"))

        # Established by Law HO-142 of May 21, 2011.
        if self._year >= 2012:
            # Family Day.
            self._add_holiday_may_15(tr("Ընտանիքի օր"))

        # Established by Law HO-187 of Dec 30, 2015.
        if self._year >= 2016:
            # Students and Youth Day.
            self._add_holiday_may_16(tr("Ուսանողների եւ երիտասարդների օր"))

        # Established by Law HO-200 of Jun 24, 2001.
        if self._year >= 2002:
            # Children's Rights Protection Day.
            self._add_childrens_day(tr("Երեխաների իրավունքների պաշտպանության օր"))

        # Established by Law HO-100 of Jun 21, 2006.
        if self._year >= 2007:
            # Day of Remembrance of the Repressed.
            self._add_holiday_jun_14(tr("Բռնադատվածների հիշատակի օր"))

        # Established by Law HO-370-N of Nov 7, 2024.
        if self._year >= 2025:
            # Father's Day.
            self._add_holiday_jun_17(tr("Հայրերի օր"))

        # Established by Law HO-228 of Oct 11, 2001.
        if self._year >= 2002:
            # Feast of Holy Etchmiadzin.
            self._add_holiday_63_days_past_easter(tr("Սուրբ Էջմիածնի տոն"))

        # Established by Law HO-67 of Mar 23, 2017.
        if self._year >= 2017:
            # State Symbols' Day.
            self._add_holiday_jul_5(tr("Պետական խորհրդանիշների օր"))

        # Established by Law HO-147-N of May 2, 2024.
        if self._year >= 2024:
            self._add_holiday_aug_3(
                # Day of Remembrance of the Victims of the Sinjar Yazidi Genocide of 2014.
                tr("2014 թվականի՝ Սինջարի եզդիների ցեղասպանության զոհերի հիշատակի օր")
            )

        # Established by Law HO-200 of Jun 24, 2001.
        # Renamed by Law HO-80 of May 2, 2005.
        if self._year >= 2001:
            self._add_holiday_sep_1(
                # Knowledge and Literacy Day.
                tr("Գիտելիքի եւ դպրության օր")
                if self._year >= 2005
                # Knowledge, Writing and Literacy Day.
                else tr("Գիտելիքի, գրի եւ դպրության օր")
            )

        # Established by Law HO-82-N of Mar 15, 2021.
        if self._year >= 2021:
            self._add_holiday_1st_sun_of_oct(
                # Day of National Minorities of the Republic of Armenia.
                tr("Հայաստանի Հանրապետության ազգային փոքրամասնությունների օր")
            )

        # Established by Law HO-134 of Oct 22, 2010.
        if self._year >= 2011:
            # Teacher's Day.
            self._add_holiday_oct_5(tr("Ուսուցչի օր"))

        # Established by Law HO-200 of Jun 24, 2001.
        if self._year >= 2001:
            # Translators' Day.
            self._add_holiday_2nd_sat_of_oct(tr("Թարգմանչաց տոն"))

        # Established by Law HO-127 of May 6, 2011.
        if self._year >= 2011:
            # Local Self-Government Day.
            self._add_holiday_nov_10(tr("Տեղական ինքնակառավարման օր"))

        # Established by Law HO-200 of Jun 24, 2001.
        # Renamed by Law HO-191-N of Nov 13, 2019.
        if self._year >= 2001:
            self._add_holiday_dec_7(
                # Earthquake Victims Remembrance Day and Disaster Resilience Day.
                tr("Երկրաշարժի զոհերի հիշատակի եւ աղետներին դիմակայունության օր")
                if self._year >= 2020
                # Earthquake Victims Remembrance Day.
                else tr("Երկրաշարժի զոհերի հիշատակի օր")
            )

        # Established by Law HO-15 of Apr 15, 2015.
        if self._year >= 2015:
            # Day of the Condemnation and Prevention of Genocides.
            self._add_holiday_dec_9(tr("Ցեղասպանությունների դատապարտման եւ կանխարգելման օր"))


class AM(Armenia):
    pass


class ARM(Armenia):
    pass


class ArmeniaStaticHolidays:
    """Armenia special holidays.

    Substituted holidays references:
        * [1998](https://web.archive.org/web/20260228163214/https://www.arlis.am/hy/acts/6981)
        * [2000](https://web.archive.org/web/20260228163311/https://www.arlis.am/hy/acts/7838)
        * [2003](https://web.archive.org/web/20260227001546/https://www.arlis.am/hy/acts/10474)
        * 2005:
            * [1](https://web.archive.org/web/20260228190830/https://www.arlis.am/hy/acts/13224)
            * [2](https://web.archive.org/web/20260228161934/https://www.arlis.am/hy/acts/13452)
            * [3](https://web.archive.org/web/20260228162103/https://www.arlis.am/hy/acts/13932)
        * 2006:
            * [1](https://web.archive.org/web/20260228162230/https://www.arlis.am/hy/acts/21549)
            * [2](https://web.archive.org/web/20260228162135/https://www.arlis.am/hy/acts/23593)
        * 2007:
            * [1](https://web.archive.org/web/20260228162206/https://www.arlis.am/hy/acts/29581)
            * [2](https://web.archive.org/web/20260228162223/https://www.arlis.am/hy/acts/34755)
        * 2008:
            * [1](https://web.archive.org/web/20260228162236/https://www.arlis.am/hy/acts/40452)
            * [2](https://web.archive.org/web/20260228162321/https://www.arlis.am/hy/acts/42730)
            * [3](https://web.archive.org/web/20260228162327/https://www.arlis.am/hy/acts/44770)
            * [4](https://web.archive.org/web/20260228162353/https://www.arlis.am/hy/acts/45880)
            * [5](https://web.archive.org/web/20260228162415/https://www.arlis.am/hy/acts/46224)
        * 2009:
            * [1](https://web.archive.org/web/20260228190831/https://www.arlis.am/hy/acts/48427)
            * [2](https://web.archive.org/web/20260320221446/https://www.arlis.am/hy/acts/48654)
            * [3](https://web.archive.org/web/20260228190925/https://www.arlis.am/hy/acts/50635)
        * 2010:
            * [1](https://web.archive.org/web/20260228162913/https://www.arlis.am/hy/acts/55456)
            * [2](https://web.archive.org/web/20260228162942/https://www.arlis.am/hy/acts/55457)
            * [3](https://web.archive.org/web/20260228163040/https://www.arlis.am/hy/acts/60567)
        * 2011:
            * [1](https://web.archive.org/web/20260228163102/https://www.arlis.am/hy/acts/60996)
            * [2](https://web.archive.org/web/20260228163117/https://www.arlis.am/hy/acts/65969)
            * [3](https://web.archive.org/web/20260228163145/https://www.arlis.am/hy/acts/69096)
        * 2012:
            * [1](https://web.archive.org/web/20260228163102/https://www.arlis.am/hy/acts/60996)
            * [2](https://web.archive.org/web/20260228163243/https://www.arlis.am/hy/acts/74278)
            * [3](https://web.archive.org/web/20260228163249/https://www.arlis.am/hy/acts/75441)
            * [4](https://web.archive.org/web/20260228190932/https://www.arlis.am/hy/acts/76805)
        * 2013:
            * [1](https://web.archive.org/web/20260228163323/https://www.arlis.am/hy/acts/80097)
            * [2](https://web.archive.org/web/20260228163334/https://www.arlis.am/hy/acts/82971)
            * [3](https://web.archive.org/web/20260320221500/https://www.arlis.am/hy/acts/83282)
        * [2013-2014](https://web.archive.org/web/20260228163443/https://www.arlis.am/hy/acts/86936)
        * 2015:
            * [1](https://web.archive.org/web/20260228163509/https://www.arlis.am/hy/acts/94257)
            * [2](https://web.archive.org/web/20260228163521/https://www.arlis.am/hy/acts/96650)
            * [3](https://web.archive.org/web/20260228163545/https://www.arlis.am/hy/acts/96794)
            * [4](https://web.archive.org/web/20260228163712/https://www.arlis.am/hy/acts/98926)
            * [5](https://web.archive.org/web/20260228163626/https://www.arlis.am/hy/acts/99539)
        * 2016:
            * [1](https://web.archive.org/web/20260227001423/https://www.arlis.am/hy/acts/103890)
            * [2](https://web.archive.org/web/20260227001500/https://www.arlis.am/hy/acts/104324)
            * [3](https://web.archive.org/web/20260227001718/https://www.arlis.am/hy/acts/108088)
        * 2017:
            * [1](https://web.archive.org/web/20260227001755/https://www.arlis.am/hy/acts/112733)
            * [2](https://web.archive.org/web/20260227001836/https://www.arlis.am/hy/acts/112882)
            * [3](https://web.archive.org/web/20260227001859/https://www.arlis.am/hy/acts/115584)
        * 2018:
            * [1](https://web.archive.org/web/20260227001920/https://www.arlis.am/hy/acts/119427)
            * [2](https://web.archive.org/web/20260227001940/https://www.arlis.am/hy/acts/120476)
            * [3](https://web.archive.org/web/20260227002118/https://www.arlis.am/hy/acts/121651)
            * [4](https://web.archive.org/web/20260227002244/https://www.arlis.am/hy/acts/125762)
            * [5](https://web.archive.org/web/20260228161847/https://www.arlis.am/hy/acts/126944)
        * 2020:
            * [1](https://web.archive.org/web/20260228162022/https://www.arlis.am/hy/acts/138403)
            * [2](https://web.archive.org/web/20260228162041/https://www.arlis.am/hy/acts/139280)
        * [2021](https://web.archive.org/web/20260228162119/https://www.arlis.am/hy/acts/155821)
    """

    # Substituted date format.
    substituted_date_format = tr("%d.%m.%Y")
    # Day off (substituted from %s).
    substituted_label = tr("Հանգստյան օր (հետաձգվել է %s թվականից)")

    special_public_holidays = {
        1998: (DEC, 7, DEC, 12),
        2000: (MAY, 8, MAY, 6),
        2003: (JAN, 3, DEC, 29, 2002),
        2005: (
            (JAN, 3, JAN, 8),
            (MAR, 7, MAR, 5),
            (JUL, 4, JUL, 2),
        ),
        2006: (
            (JAN, 3, JAN, 8),
            (MAY, 8, MAY, 6),
        ),
        2007: (
            (JAN, 3, DEC, 30, 2006),
            (JAN, 4, JAN, 7),
            (APR, 30, APR, 28),
        ),
        2008: (
            (JAN, 3, DEC, 29, 2007),
            (JAN, 4, JAN, 12),
            (MAY, 2, MAY, 4),
            (JUN, 30, JUN, 28),
            (AUG, 18, AUG, 16),
            (SEP, 15, SEP, 13),
        ),
        2009: (
            (JAN, 5, JAN, 10),
            (JAN, 7, DEC, 27, 2008),
            (APR, 13, APR, 18),
            (JUL, 20, JUL, 18),
            (AUG, 17, AUG, 15),
            (SEP, 14, SEP, 12),
        ),
        2010: (
            (JAN, 8, DEC, 26, 2009),
            (APR, 5, APR, 10),
            (JUL, 12, JUL, 17),
            (AUG, 16, AUG, 21),
            (SEP, 13, SEP, 18),
            (SEP, 20, SEP, 11),
        ),
        2011: (
            (MAR, 7, MAR, 12),
            (APR, 25, APR, 30),
            (JUL, 4, JUL, 9),
            (AUG, 1, AUG, 6),
            (AUG, 15, AUG, 20),
            (SEP, 12, SEP, 17),
        ),
        2012: (
            (MAR, 9, MAR, 24),
            (APR, 9, APR, 14),
            (APR, 30, APR, 28),
            (JUL, 6, JUL, 14),
            (JUL, 16, JUL, 21),
            (AUG, 13, AUG, 18),
            (SEP, 17, SEP, 29),
        ),
        2013: (
            (APR, 1, APR, 13),
            (MAY, 10, MAY, 18),
            (MAY, 27, JUN, 1),
            (JUL, 8, JUL, 13),
            (AUG, 19, AUG, 24),
            (SEP, 16, SEP, 28),
            (DEC, 30, DEC, 28),
        ),
        2014: (
            (JAN, 27, FEB, 1),
            (APR, 21, APR, 26),
            (MAY, 2, MAY, 31),
            (JUL, 28, AUG, 2),
            (AUG, 18, AUG, 23),
            (SEP, 15, SEP, 27),
        ),
        2015: (
            (JAN, 8, DEC, 20, 2014),
            (JAN, 9, DEC, 27, 2014),
            (APR, 6, APR, 11),
            (APR, 23, APR, 18),
            (JUL, 13, JUL, 18),
            (SEP, 14, SEP, 19),
        ),
        2016: (
            (MAR, 7, MAR, 12),
            (MAR, 28, APR, 2),
            (SEP, 12, SEP, 24),
        ),
        2017: (
            (APR, 17, MAY, 6),
            (MAY, 8, MAY, 20),
            (SEP, 18, SEP, 23),
        ),
        2018: (
            (MAR, 9, MAR, 17),
            (APR, 2, APR, 7),
            (APR, 30, MAY, 5),
            (OCT, 11, OCT, 27),
            (OCT, 12, NOV, 3),
        ),
        2020: (
            (JAN, 27, FEB, 1),
            (MAY, 29, MAY, 23),
        ),
        2021: (SEP, 20, SEP, 25),
    }

    special_lo_public_holidays = {
        2018: (DEC, 7, DEC, 15),
    }

    special_sh_public_holidays = {
        2018: (DEC, 7, DEC, 15),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, APR, JUN, JUL, AUG, SEP, NOV, DEC
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Chad(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Chad holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Chad>
        * <https://web.archive.org/web/20240619220557/https://www.ilo.org/dyn/natlex/docs/ELECTRONIC/97323/115433/F-316075167/TCD-97323.pdf>
    """

    country = "TD"
    # %s (estimated).
    estimated_label = "%s (estimated)"
    # %s (observed, estimated).
    observed_estimated_label = "%s (observed, estimated)"
    # %s (observed).
    observed_label = "%s (observed)"
    # On 11 August 1960, Chad gained independence from France.
    start_year = 1961

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=ChadIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, ChadStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # International Women's Day.
        self._add_observed(self._add_womens_day("International Women's Day"))

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Labour Day.
        self._add_observed(self._add_labor_day("Labour Day"))

        # Independence Day.
        self._add_observed(self._add_holiday_aug_11("Independence Day"))

        # All Saints' Day.
        self._add_all_saints_day("All Saints' Day")

        # Republic Day.
        self._add_observed(self._add_holiday_nov_28("Republic Day"))

        if self._year >= 1991:
            # Freedom and Democracy Day.
            self._add_observed(self._add_holiday_dec_1("Freedom and Democracy Day"))

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Eid al-Fitr.
        self._add_eid_al_fitr_day("Eid al-Fitr")

        # Eid al-Adha.
        self._add_eid_al_adha_day("Eid al-Adha")

        # Mawlid.
        self._add_mawlid_day("Mawlid")


class TD(Chad):
    pass


class TCD(Chad):
    pass


class ChadIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2015, 2023)
    EID_AL_ADHA_DATES = {
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2015, 2024)
    EID_AL_FITR_DATES = {
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2015, 2022)
    MAWLID_DATES = {
        2015: ((JAN, 3), (DEC, 24)),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2018: (NOV, 21),
    }


class ChadStaticHolidays:
    special_public_holidays = {
        2021: (APR, 23, "Funeral of Idriss Déby Itno"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, DEC, SUN
from holidays.constants import GOVERNMENT, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Paraguay(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Paraguay holidays.

    References:
        * [Ley 8/1990](https://web.archive.org/web/20250427173904/https://www.bacn.gov.py/leyes-paraguayas/2358/ley-n-8--por-la-cual-se-determinan-los-feriados-de-la-republica)
        * [Ley 715/1995](https://web.archive.org/web/20250427174040/https://www.bacn.gov.py/leyes-paraguayas/792/ley-n-715--amplia-la-ley-no-0890-)
        * [Ley 1.601/2000](https://web.archive.org/web/20250427173847/https://www.bacn.gov.py/leyes-paraguayas/1677/ley-n-1601--modifica-el-articulo-1-de-la-ley-no-71595-que-amplia-la-ley-n-0890-por-la-cual-se-determinan-los-feriados-de-la-republica)
        * [Ley 1.723/2001](https://web.archive.org/web/20250427173914/https://www.bacn.gov.py/leyes-paraguayas/634/ley-n-1723--autoriza-al-poder-ejecutivo-a-trasladar-los-feriados-nacionales-al-dia-lunes)
        * [Ley 4.531/2011](https://web.archive.org/web/20250420210317/https://www.bacn.gov.py/leyes-paraguayas/3831/ley-n-4531-restablece-el-dia-14-de-mayo-de-cada-ano-como-feriado-nacional)
        * <https://es.wikipedia.org/wiki/Anexo:Días_festivos_en_Paraguay>
        * <https://web.archive.org/web/20131212214628/https://www.abc.com.py/nacionales/feriados-de-2013-se-trasladaran-al-dia-lunes-494705.html>
        * <https://web.archive.org/web/20241207131557/https://www.ultimahora.com/establecen-traslados-feriados-el-2017-n1042098>
        * <https://web.archive.org/web/20250913135012/https://foco.lanacion.com.py/2025/01/01/feriados-2025-paraguay-tendra-6-fines-de-semana-largos/>
    """

    country = "PY"
    default_language = "es"
    supported_categories = (GOVERNMENT, PUBLIC)
    supported_languages = ("en_US", "es", "uk")
    # Ley 8/1990.
    start_year = 1991
    weekend = {SUN}

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, ParaguayStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        dates_obs = {
            2013: (MAR, 4),
            2016: (FEB, 29),
            2017: (FEB, 27),
            2018: (FEB, 26),
            2019: (MAR, 4),
            2022: (FEB, 28),
            2023: (FEB, 27),
            2025: (MAR, 3),
        }
        self._add_holiday(
            # Patriots Day.
            tr("Día de los Héroes de la Patria"),
            dates_obs.get(self._year, (MAR, 1)),
        )

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jueves Santo"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Domingo de Resurrección"))

        # Workers' Day.
        self._add_labor_day(tr("Día de los Trabajadores"))

        if self._year >= 2012:
            # National Holiday.
            self._add_holiday_may_14(tr("Feriado Nacional"))

        # Independence Day.
        self._add_holiday_may_15(tr("Día de la Independencia Nacional"))

        dates_obs = {
            2013: (JUN, 10),
            2014: (JUN, 16),
            2018: (JUN, 11),
            2019: (JUN, 17),
            2024: (JUN, 10),
            2025: (JUN, 16),
        }
        # Chaco Armistice Day.
        self._add_holiday(tr("Día de la Paz del Chaco"), dates_obs.get(self._year, (JUN, 12)))

        # Asuncion Foundation's Day.
        name = tr("Día de la Fundación de Asunción")
        # Decreto N° 6292.
        if self._year == 2017:
            self._add_holiday_aug_14(name)
        else:
            self._add_holiday_aug_15(name)

        # Ley 715/1995, 1.601/2000.
        if self._year >= 1995:
            dates_obs = {
                2015: (SEP, 28),
                2016: (OCT, 3),
                2017: (OCT, 2),
                2021: (SEP, 27),
                2022: (OCT, 3),
                2024: (SEP, 30),
            }
            self._add_holiday(
                # Boqueron Battle Day.
                tr("Día de la Batalla de Boquerón"),
                dates_obs.get(self._year, (SEP, 29)),
            )

        # Caacupe Virgin Day.
        self._add_holiday_dec_8(tr("Día de la Virgen de Caacupé"))

        # Christmas Day.
        self._add_christmas_day(tr("Día de la Navidad"))


class PY(Paraguay):
    pass


class PRY(Paraguay):
    pass


class ParaguayStaticHolidays:
    """Paraguay special holidays.

    References:
        * <https://web.archive.org/web/20250917075450/https://www.pgr.gov.py/wp-content/uploads/2013/12/DECRETO-991-ASUETO-EL-24-Y-EL-31-DE-DICIEMBRE.pdf>
        * <https://web.archive.org/web/20250917074759/https://www.ultimahora.com/ejecutivo-declara-asueto-el-23-y-30-diciembre-sector-publico-n3040041>
        * <https://web.archive.org/web/20231003173505/https://www.lanacion.com.py/politica/2023/03/24/decretan-asueto-de-funcionarios-publicos-el-miercoles-5-de-abril-por-semana-santa/>
        * <https://web.archive.org/web/20250917042124/https://informacionpublica.paraguay.gov.py/public/1046579-RESPUESTAAIPN79397pdf-RESPUESTAAIPN79397.pdf>
        * <https://web.archive.org/web/20250416092842/https://www.radionacional.gov.py/2024/03/22/ejecutivo-declara-asueto-el-miercoles-27-de-marzo-para-funcionarios-publicos/>
        * <https://web.archive.org/web/20241225185947/https://www.abc.com.py/politica/2024/12/20/declaran-asueto-para-funcionarios-publicos-el-24-y-31-de-diciembre/>
        * <https://web.archive.org/web/20250917075032/https://portal.ips.gov.py/sistemas/ipsportal/archivos/archivos/1744391513.pdf>
    """

    # Public holiday.
    public_holiday = tr("Asueto adicional")
    # Public sector holiday.
    public_sector_holiday = tr("Asueto de la Administración Pública")

    special_public_holidays = {
        # Public holiday for business purposes, in view of the recently increased risk
        # of Dengue fever.
        2007: (JAN, 29, public_holiday),
        # Public sector holiday to celebrate Paraguay's football team's qualification
        # for the 2010 World Cup.
        2009: (SEP, 10, public_holiday),
        # Public holiday to coincide with the Paraguay-Italy game of the current World Cup.
        2010: (JUN, 14, public_holiday),
        2011: (
            # Public holiday to coincide with the current anti-Dengue drive.
            (APR, 19, public_holiday),
            # Public holidays to commemorate the Bicentennial of Paraguay's independence.
            (MAY, 14, public_holiday),
            (MAY, 16, public_holiday),
        ),
        # Date of the inauguration of President-elect Horacio Cartes.
        2013: (AUG, 14, public_holiday),
        2015: (
            # Public holidays in Paraguay on account of the upcoming visit of Pope Francis
            # in Paraguay.
            (JUL, 10, public_holiday),
            (JUL, 11, public_holiday),
        ),
    }

    special_government_holidays = {
        2010: (
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2011: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 20, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 23, public_sector_holiday),
            (DEC, 30, public_sector_holiday),
        ),
        2012: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 4, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2013: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (MAR, 27, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2014: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 16, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2015: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 1, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        # Public sector holiday to let civil servants begin their Holy Week earlier.
        2016: (MAR, 23, public_sector_holiday),
        # Public sector holiday to let civil servants begin their Holy Week earlier.
        2017: (MAR, 28, public_sector_holiday),
        2018: (
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2019: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 17, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        # Public sector holiday to let civil servants begin their Holy Week earlier.
        2020: (APR, 8, public_sector_holiday),
        2021: (
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        2022: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 13, public_sector_holiday),
            # Public sector holiday due to the annual May 1st public holiday falling on a Sunday.
            (MAY, 2, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 23, public_sector_holiday),
            (DEC, 30, public_sector_holiday),
        ),
        2023: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (APR, 5, public_sector_holiday),
            # Public sector holiday from resolution no. 1045.
            (DEC, 7, public_sector_holiday),
        ),
        2024: (
            # Public sector holiday to let civil servants begin their Holy Week earlier.
            (MAR, 27, public_sector_holiday),
            # 2 year-end public sector holidays.
            (DEC, 24, public_sector_holiday),
            (DEC, 31, public_sector_holiday),
        ),
        # Public sector holiday to let civil servants begin their Holy Week earlier.
        2025: (APR, 16, public_sector_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import FEB
from holidays.constants import PUBLIC
from holidays.countries.china import China, ChinaStaticHolidays
from holidays.groups import ChineseCalendarHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class ShanghaiStockExchange(China, ChinaStaticHolidays):
    """Shanghai Stock Exchange (SSE) holidays.

    This class provides Shanghai Stock Exchange-specific market holidays.
    Market holidays are days when the stock exchange is closed for trading.

    References:
        * <https://web.archive.org/web/20260129171258/http://english.sse.com.cn/start/trading/schedule/>

    Historical data:
        * [2020](https://web.archive.org/web/20250620222003/http://www.sse.com.cn/disclosure/announcement/general/c/c_20191220_4969627.shtml)
        * [2020 changes](https://web.archive.org/web/20200211063350/http://www.sse.com.cn:80/star/media/news/c/c_20200127_4991583.shtml)
        * [2021](https://web.archive.org/web/20260212161241/http://www.sse.com.cn/disclosure/announcement/general/c/c_20201224_5286949.shtml)
        * [2022](https://web.archive.org/web/20220518224459/http://www.sse.com.cn/disclosure/announcement/general/c/c_20211220_5662606.shtml)
        * [2023](https://web.archive.org/web/20251213130614/http://www.sse.com.cn/disclosure/announcement/general/c/c_20221227_5714458.shtml)
        * [2024](https://web.archive.org/web/20251208095335/http://www.sse.com.cn/disclosure/announcement/general/c/c_20231226_5733939.shtml)
        * [2025](https://web.archive.org/web/20251213122921/https://www.sse.com.cn/disclosure/announcement/general/c/c_20241223_10767108.shtml)
    """

    country = None  # type: ignore[assignment]
    market = "XSHG"
    parent_entity = China
    supported_categories: tuple[str, ...] = (PUBLIC,)  # type: ignore[assignment]
    start_year = 2001

    def __init__(self, *args, **kwargs):
        ChineseCalendarHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=ShanghaiStockExchangeStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2000)
        ObservedHolidayBase.__init__(self, *args, **kwargs)

    def _populate_common_holidays(self):
        super()._populate_common_holidays()

        # SSE keeps national makeup work weekends closed and publishes its own
        # holiday calendar instead of inheriting China weekend workdays.
        self.weekend_workdays.clear()


class XSHG(ShanghaiStockExchange):
    pass


class SSE(ShanghaiStockExchange):
    pass


class ShanghaiStockExchangeStaticHolidays:
    special_public_holidays = {
        **ChinaStaticHolidays.special_public_holidays,
        2024: (
            *ChinaStaticHolidays.special_public_holidays[2024],
            # Chinese New Year's Eve.
            (FEB, 9, tr("农历除夕")),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.groups import InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Pakistan(HolidayBase, InternationalHolidays, IslamicHolidays):
    """Pakistan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Pakistan>
        * <https://ur.wikipedia.org/wiki/تعطیلات_پاکستان>
        * [Public and optional holidays](https://web.archive.org/web/20250118105814/https://cabinet.gov.pk/Detail/OTE2ODBiYmItZmI0MS00NDAwLWE5NGUtYmE1MGVjYzllMzAz)
        * [No.10-01/2024-Min-II](https://web.archive.org/web/20241103080918/https://cabinet.gov.pk/SiteImage/Misc/files/Holidays/28-5-24.pdf)
    """

    country = "PK"
    default_language = "en_PK"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # Achieved Independence on August 14th, 1947.
    start_year = 1948
    supported_languages = ("en_PK", "en_US", "ur_PK")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=PakistanIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 1990:
            # Kashmir Solidarity Day.
            self._add_holiday_feb_5(tr("Kashmir Solidarity Day"))

        if self._year >= 1956:
            # Pakistan Day.
            self._add_holiday_mar_23(tr("Pakistan Day"))

        if self._year >= 1972:
            # Labor Day.
            self._add_labor_day(tr("Labour Day"))

        if self._year >= 2024:
            # Youm-e-Takbeer.
            self._add_holiday_may_28(tr("Youm-e-Takbeer"))

        # Independence Day.
        self._add_holiday_aug_14(tr("Independence Day"))

        if self._year <= 2014 or self._year >= 2022:
            # Iqbal Day.
            self._add_holiday_nov_9(tr("Iqbal Day"))

        # Quaid-e-Azam Day.
        self._add_holiday_dec_25(tr("Quaid-e-Azam Day"))

        # Eid al-Fitr.
        name = tr("Eid-ul-Fitr")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("Eid-ul-Adha")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)

        # Prophet's Birthday.
        self._add_mawlid_day(tr("Eid Milad-un-Nabi"))

        # Ashura.
        name = tr("Ashura")
        self._add_ashura_eve(name)
        self._add_ashura_day(name)


class PK(Pakistan):
    pass


class PAK(Pakistan):
    pass


class PakistanIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20250724060822/https://www.timeanddate.com/holidays/pakistan/ashura
    ASHURA_DATES_CONFIRMED_YEARS = (2005, 2025)
    ASHURA_DATES = {
        2005: (FEB, 18),
        2006: (FEB, 8),
        2007: (JAN, 28),
        2008: (JAN, 18),
        2009: ((JAN, 6), (DEC, 26)),
        2012: (NOV, 23),
        2018: (SEP, 21),
        2022: (AUG, 9),
        2025: (JUL, 6),
    }

    # https://web.archive.org/web/20250724060629/https://www.timeanddate.com/holidays/pakistan/eid-ul-azha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2005, 2025)
    EID_AL_ADHA_DATES = {
        2008: (DEC, 9),
        2009: (NOV, 28),
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 6),
        2015: (SEP, 24),
        2016: (SEP, 12),
        2017: (SEP, 2),
        2018: (AUG, 22),
        2019: (AUG, 12),
        2021: (JUL, 21),
        2022: (JUL, 10),
        2023: (JUN, 29),
        2024: (JUN, 17),
        2025: (JUN, 7),
    }

    # https://web.archive.org/web/20250724060834/https://www.timeanddate.com/holidays/pakistan/eid-ul-fitr-1
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2005, 2025)
    EID_AL_FITR_DATES = {
        2005: (NOV, 4),
        2006: (OCT, 24),
        2008: (OCT, 2),
        2009: (SEP, 21),
        2011: (AUG, 31),
        2014: (JUL, 29),
        2017: (JUN, 26),
        2018: (JUN, 16),
        2019: (JUN, 5),
        2022: (MAY, 3),
        2023: (APR, 22),
        2025: (MAR, 31),
    }

    # https://web.archive.org/web/20250724061027/https://www.timeanddate.com/holidays/pakistan/eid-milad-un-nabi

    MAWLID_DATES_CONFIRMED_YEARS = (2005, 2024)
    MAWLID_DATES = {
        2005: (APR, 22),
        2006: (APR, 11),
        2008: (MAR, 21),
        2010: (MAR, 1),
        2011: (FEB, 17),
        2012: (FEB, 5),
        2014: (JAN, 14),
        2015: (JAN, 4),
        2016: (DEC, 12),
        2017: (DEC, 1),
        2018: (NOV, 21),
        2019: (NOV, 10),
        2020: (OCT, 30),
        2021: (OCT, 19),
        2022: (OCT, 9),
        2023: (SEP, 29),
        2024: (SEP, 17),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, APR, MAY, JUL, AUG, SEP, MON, TUE, WED, FRI, SAT, SUN
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.constants import GOVERNMENT, PUBLIC, SCHOOL
from holidays.groups import (
    ChristianHolidays,
    IslamicHolidays,
    InternationalHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, ObservedRule

EG_OBSERVED_RULE = ObservedRule({MON: +3, TUE: +2, WED: +1, SUN: +4})


class Egypt(
    ObservedHolidayBase, ChristianHolidays, IslamicHolidays, InternationalHolidays, StaticHolidays
):
    """Egypt holidays.

    References:
        * <https://ar.wikipedia.org/wiki/قائمة_العطل_الرسمية_في_مصر>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Egypt>
        * [National Holidays (Arabic)](https://web.archive.org/web/20250614072551/https://www.presidency.eg/ar/مصر/العطلات-الرسمية/)
        * [National Holidays (English)](https://web.archive.org/web/20250529043734/https://www.presidency.eg/en/مصر/العطلات-الرسمية/)
        * [National Holidays (French)](https://web.archive.org/web/20250608173134/https://www.presidency.eg/fr/مصر/العطلات-الرسمية/)
        * [Ministerial Decision 1193](https://web.archive.org/web/20250423073350/https://manshurat.org/node/44922)
        * [2026 Eid al-Fitr](https://web.archive.org/web/20260331062747/https://sis.gov.eg/en/media-center/news/dar-al-ifta-announces-friday-as-the-first-day-of-eidul-fitr/)
    """

    country = "EG"
    default_language = "ar_EG"
    # %s (estimated).
    estimated_label = tr("%s (تقديري)")
    # %s (observed).
    observed_label = tr("%s (يوم تعويضي)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (يوم تعويضي تقديري)")
    # Republic of Egypt was declared on 18 June 1953.
    start_year = 1954
    supported_categories = (GOVERNMENT, PUBLIC, SCHOOL)
    supported_languages = ("ar_EG", "en_US", "fr")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=EgyptIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, cls=EgyptStaticHolidays)
        kwargs.setdefault("observed_rule", EG_OBSERVED_RULE)
        kwargs.setdefault("observed_since", 2020)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 2002 and self._year not in {2022, 2023}:
            # Coptic Christmas Day.
            self._add_christmas_day(tr("عيد الميلاد المجيد"))

        if self._year >= 2009:
            self._move_holiday(
                self._add_holiday_jan_25(
                    # January 25th Revolution and National Police Day.
                    tr("ثورة ٢٥ يناير وعيد الشرطة")
                    if self._year >= 2012
                    # National Police Day.
                    else tr("عيد الشرطة")
                )
            )

        if self._year >= 1983:
            # Sinai Liberation Day.
            dt = self._add_holiday_apr_25(tr("عيد تحرير سيناء"))
            if self._year != 2022:
                self._move_holiday(dt)

        # Spring Festival.
        self._add_easter_monday(tr("عيد شم النسيم"))

        # Labor Day.
        if self._year != 2024:
            self._move_holiday(self._add_labor_day(tr("عيد العمال")))

        if self._year >= 2014:
            # June 30 Revolution Day.
            dt = self._add_holiday_jun_30(tr("عيد ثورة ٣٠ يونيو"))
            if self._year != 2024:
                self._move_holiday(dt)

        # July 23 Revolution Day.
        dt = self._add_holiday_jul_23(tr("عيد ثورة ٢٣ يوليو"))
        if self._year != 2023:
            self._move_holiday(dt)

        # Armed Forces Day.
        dt = self._add_holiday_oct_6(tr("عيد القوات المسلحة"))
        if self._year != 2024:
            self._move_holiday(dt)

        # Islamic New Year.
        for dt in self._add_islamic_new_year_day(tr("رأس السنة الهجرية")):
            self._move_holiday(dt)

        # Prophet's Birthday.
        self._add_mawlid_day(tr("المولد النبوي الشريف"))

        # Eid al-Fitr.
        name = tr("عيد الفطر المبارك")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)

        # Arafat Day.
        self._add_arafah_day(tr("وقفة عيد الأضحى المبارك"))

        # Eid al-Adha.
        name = tr("عيد الأضحى المبارك")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)

    def _populate_government_holidays(self):
        if self._year >= 2019:
            # Eid al-Fitr.
            name = tr("عيد الفطر المبارك")
            self._add_eid_al_fitr_eve(name)
            self._add_eid_al_fitr_day_three(name)

        if self._year >= 2018:
            # Eid al-Adha.
            self._add_eid_al_adha_day_four(tr("عيد الأضحى المبارك"))

    def _populate_school_holidays(self):
        if self._year >= 2019:
            # Taba Liberation Day.
            self._add_holiday_mar_19(tr("عيد تحرير طابا"))

            # Evacuation Day.
            self._add_holiday_jun_18(tr("عيد الجلاء"))


class EG(Egypt):
    pass


class EGY(Egypt):
    pass


class EgyptIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2020, 2025)
    EID_AL_ADHA_DATES = {
        2022: (JUL, 10),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2020, 2026)
    EID_AL_FITR_DATES = {
        2022: (MAY, 1),
    }

    HIJRI_NEW_YEAR_DATES_CONFIRMED_YEARS = (2020, 2025)
    HIJRI_NEW_YEAR_DATES = {
        2021: (AUG, 11),
    }

    MAWLID_DATES_CONFIRMED_YEARS = (2020, 2025)
    MAWLID_DATES = {
        2023: (SEP, 28),
    }


class EgyptStaticHolidays:
    """Egypt special holidays.

    References:
        * [2022 Coptic Christmas](https://web.archive.org/web/20240227004025/https://english.ahram.org.eg/NewsContent/1/2/454491/Egypt/Society/Thursday-paid-day-off-at-public-sector-in-Egypt-fo.aspx)
    """

    # Coptic Christmas Day.
    coptic_christmas_day = tr("عيد الميلاد المجيد")

    # Eid al-Adha.
    eid_al_adha = tr("عيد الأضحى المبارك")

    # Eid al-Fitr.
    eid_al_fitr = tr("عيد الفطر المبارك")

    special_public_holidays = {
        2021: (
            (JUL, 17, eid_al_adha),
            (JUL, 18, eid_al_adha),
        ),
        2022: (
            (JUL, 13, eid_al_adha),
            (JUL, 14, eid_al_adha),
        ),
    }

    special_public_holidays_observed = {
        2022: (
            (JAN, 6, coptic_christmas_day),
            (MAY, 4, eid_al_fitr),
        ),
        2023: (
            (JAN, 8, coptic_christmas_day),
            (APR, 24, eid_al_fitr),
            # June 30 Revolution Day.
            (JUL, 2, tr("عيد ثورة ٣٠ يونيو")),
            (JUL, 3, eid_al_adha),
        ),
        # Labor Day.
        2024: (MAY, 5, tr("عيد العمال")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import _get_all_sundays
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Norway(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Norway holidays.

    References:
        * <https://web.archive.org/web/20250102052441/https://lovdata.no/dokument/NL/lov/1947-04-26-1>
        * <https://no.wikipedia.org/wiki/Helligdager_i_Norge>
        * <https://web.archive.org/web/20250416120557/https://www.timeanddate.no/merkedag/norge>

    Note that holidays falling on a Sunday are "lost"; they will not be moved
    to another day to make up for the collision.

    In Norway, [all Sundays are considered holidays](https://web.archive.org/web/20250202193959/https://snl.no/helligdag).
    Initialize this class with `include_sundays=False` to not include Sundays as holidays.
    """

    country = "NO"
    default_language = "no"
    subdivisions: tuple[str, ...] = (
        "03",  # Oslo.
        "11",  # Rogaland.
        "15",  # Møre og Romsdal.
        "18",  # Nordland.
        "21",  # Svalbard.
        "22",  # Jan Mayen.
        "31",  # Østfold.
        "32",  # Akershus.
        "33",  # Buskerud.
        "34",  # Innlandet.
        "39",  # Vestfold.
        "40",  # Telemark.
        "42",  # Agder.
        "46",  # Vestland.
        "50",  # Trøndelag (Trööndelage).
        "55",  # Troms (Romssa, Tromssan).
        "56",  # Finnmark (Finnmárkku, Finmarkun).
    )
    subdivisions_aliases = {
        "Oslo": "03",
        "Rogaland": "11",
        "Møre og Romsdal": "15",
        "Nordland": "18",
        "Svalbard": "21",
        "Jan Mayen": "22",
        "Østfold": "31",
        "Akershus": "32",
        "Buskerud": "33",
        "Innlandet": "34",
        "Vestfold": "39",
        "Telemark": "40",
        "Agder": "42",
        "Vestland": "46",
        "Trööndelage": "50",
        "Trøndelag": "50",
        "Romssa": "55",
        "Troms": "55",
        "Tromssan": "55",
        "Finmarkun": "56",
        "Finnmark": "56",
        "Finnmárkku": "56",
    }
    supported_languages = ("en_US", "no", "th", "uk")
    _deprecated_subdivisions = (
        "30",
        "38",
        "54",
    )

    def __init__(self, *args, include_sundays: bool = False, **kwargs):
        """
        Args:
            include_sundays:
                Whether to consider Sundays as holidays (which they are in Norway)
        """
        self.include_sundays = include_sundays
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Første nyttårsdag"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Skjærtorsdag"))

        # Good Friday.
        self._add_good_friday(tr("Langfredag"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Første påskedag"))

        # Easter Monday.
        self._add_easter_monday(tr("Andre påskedag"))

        # Source: https://web.archive.org/web/20250102052441/https://lovdata.no/dokument/NL/lov/1947-04-26-1
        if self._year >= 1947:
            # Labor Day.
            self._add_labor_day(tr("Arbeidernes dag"))

            # Constitution Day.
            self._add_holiday_may_17(tr("Grunnlovsdag"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Kristi himmelfartsdag"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Første pinsedag"))

        # Whit Monday.
        self._add_whit_monday(tr("Andre pinsedag"))

        # According to https://no.wikipedia.org/wiki/Første_juledag,
        # these dates are only valid from year > 1700
        # Wikipedia has no source for the statement, so leaving this be for now

        # Christmas Day.
        self._add_christmas_day(tr("Første juledag"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Andre juledag"))

        if self.include_sundays:
            # Optionally add all Sundays of the year.
            for dt in _get_all_sundays(self._year):
                # Sunday.
                self._add_holiday(tr("Søndag"), dt)


class NO(Norway):
    pass


class NOR(Norway):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import MAY
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Lesotho(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Lesotho holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Lesotho>
        * [Act No. 7 of 1995](https://web.archive.org/web/20220508094900/https://www.ilo.org/dyn/travail/docs/2093/Public%20Holidays%20Act%201995.pdf)
        * <https://web.archive.org/web/20250317192347/https://www.timeanddate.com/holidays/lesotho/>
    """

    country = "LS"
    start_year = 1996

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, LesothoStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day("New Year's Day")

        # Moshoeshoe's Day.
        self._add_holiday_mar_11("Moshoeshoe's Day")

        if self._year <= 2002:
            # Heroes Day.
            self._add_holiday_apr_4("Heroes Day")

        if self._year >= 2003:
            # Africa/Heroes Day.
            self._add_africa_day("Africa/Heroes Day")

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Workers' Day.
        self._add_labor_day("Workers' Day")

        # Ascension Day.
        self._add_ascension_thursday("Ascension Day")

        # https://en.wikipedia.org/wiki/Letsie_III
        # King's Birthday.
        name = "King's Birthday"
        if self._year >= 1998:
            self._add_holiday_jul_17(name)
        else:
            self._add_holiday_may_2(name)

        # Independence Day.
        self._add_holiday_oct_4("Independence Day")

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Boxing Day.
        self._add_christmas_day_two("Boxing Day")


class LS(Lesotho):
    pass


class LSO(Lesotho):
    pass


class LesothoStaticHolidays:
    special_public_holidays = {
        2002: (MAY, 25, "Africa Day"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import SUN
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Tokelau(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Tokelau holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Tokelau>
        * <https://web.archive.org/web/20250715060109/https://www.tokelau.org.nz/About+Us/Culture.html>
        * [Interpretation Rules 2003](https://web.archive.org/web/20250715065528/https://paclii.org/tk/legis/consol_act_2016/ir2003240.pdf)
    """

    country = "TK"
    default_language = "en_TK"
    supported_languages = ("en_TK", "en_US", "tkl")
    # Interpretation Rules 2003.
    start_year = 2003
    weekend = {SUN}

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("New Year's Day"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Tokehega Day.
        self._add_holiday_sep_3(tr("Tokehega Day"))

        # Christmas Day.
        self._add_christmas_day(tr("Christmas Day"))

        # Boxing Day.
        self._add_christmas_day_two(tr("Boxing Day"))


class TK(Tokelau):
    pass


class TKL(Tokelau):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.julian_revised import JULIAN_REVISED_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Romania(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Romania holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Romania>
        * [Law No. 75 of Jul 12, 1996](https://web.archive.org/web/20260120033731/https://legislatie.just.ro/Public/DetaliiDocumentAfis/8417)
        * [Labor Code 2003](https://web.archive.org/web/20260309153044/https://legislatie.just.ro/Public/DetaliiDocumentAfis/304539)
        * [Law No. 202 of Oct 21, 2008](https://web.archive.org/web/20251207215906/https://legislatie.just.ro/Public/DetaliiDocumentAfis/98527)
        * [Law No. 147 of Jul 23, 2012](https://web.archive.org/web/20251007033324/https://legislatie.just.ro/Public/DetaliiDocumentAfis/139884)
        * [Law No. 176 of Oct 7, 2016](https://web.archive.org/web/20251011075049/https://legislatie.just.ro/Public/DetaliiDocumentAfis/182520)
        * [Law No. 220 of Nov 17, 2016](https://web.archive.org/web/20251117213440/https://legislatie.just.ro/Public/DetaliiDocumentAfis/183741)
        * [Law No. 64 of Mar 12, 2018](https://web.archive.org/web/20251018174920/https://legislatie.just.ro/Public/DetaliiDocumentAfis/198708)
        * [Law No. 52 of Mar 3, 2023](https://web.archive.org/web/20251205085234/https://legislatie.just.ro/Public/DetaliiDocumentAfis/265498)
    """

    country = "RO"
    default_language = "ro"
    # Law No. 75 of Jul 12, 1996.
    start_year = 1997
    supported_languages = ("en_US", "ro", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self, JULIAN_REVISED_CALENDAR)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Anul Nou")
        self._add_new_years_day(name)
        self._add_new_years_day_two(name)

        # Established by Law No. 52 of Mar 3, 2023.
        if self._year >= 2024:
            # Epiphany.
            self._add_epiphany_day(tr("Botezul Domnului - Boboteaza"))

            # Saint John the Baptist.
            self._add_holiday_jan_7(tr("Soborul Sfântului Proroc Ioan Botezătorul"))

        # Established by Law No. 176 of Oct 7, 2016.
        if self._year >= 2017:
            # Unification of the Romanian Principalities Day.
            self._add_holiday_jan_24(tr("Ziua Unirii Principatelor Române"))

        # Established by Law No. 64 of Mar 12, 2018.
        if self._year >= 2018:
            # Good Friday.
            self._add_good_friday(tr("Vinerea Mare"))

        # Easter.
        name = tr("Paștele")
        self._add_easter_sunday(name)
        self._add_easter_monday(name)

        # Labor Day.
        self._add_labor_day(tr("Ziua Muncii"))

        # Established by Law No. 220 of Nov 17, 2016.
        if self._year >= 2017:
            # Children's Day.
            self._add_childrens_day(tr("Ziua Copilului"))

        # Established by Law No. 202 of Oct 21, 2008.
        if self._year >= 2009:
            # Pentecost.
            name = tr("Rusaliile")
            self._add_whit_sunday(name)
            self._add_whit_monday(name)

            # Dormition of the Mother of God.
            self._add_assumption_of_mary_day(tr("Adormirea Maicii Domnului"))

        # Established by Law No. 147 of Jul 23, 2012.
        if self._year >= 2012:
            self._add_holiday_nov_30(
                # Saint Andrew's Day.
                tr("Sfântul Apostol Andrei, cel Întâi chemat, Ocrotitorul României")
            )

        # National Day.
        self._add_holiday_dec_1(tr("Ziua Națională a României"))

        # Christmas Day.
        name = tr("Crăciunul")
        self._add_christmas_day(name)
        self._add_christmas_day_two(name)


class RO(Romania):
    pass


class ROU(Romania):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class VaticanCity(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Vatican City holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Vatican_City>
        * <https://en.wikipedia.org/wiki/Holy_day_of_obligation>
        * [Regolamento generale del personale del Governatorato vaticano](https://web.archive.org/web/20250125172412/https://www.vatican.va/roman_curia/labour_office/docs/documents/ulsa_b18_7_it.html)
        * [1917 Code of Canon Law](https://web.archive.org/web/20250320055902/https://cdn.restorethe54.com/media/pdf/1917-code-of-canon-law-english.pdf)
        * [1983 Code of Canon Law](https://web.archive.org/web/20250425093046/https://www.vatican.va/archive/cod-iuris-canonici/eng/documents/cic_lib4-cann1244-1253_en.html)
        * [Solemnity of Mary, Mother of God](https://web.archive.org/web/20241004015531/https://www.ewtn.com/catholicism/library/solemnity-of-mary-mother-of-god-5826)
        * [Saint Joseph the Worker](https://web.archive.org/web/20241130180934/https://www.franciscanmedia.org/saint-of-the-day/saint-joseph-the-worker/)

    Cross-checked With:
        * [2020](https://web.archive.org/web/20250427173707/https://www.vaticanstate.va/images/pdf/CALENDARIO_2020.pdf)
        * [2021](https://web.archive.org/web/20220723175312/https://www.farmaciavaticana.va/images/pdf/calendario_2021.pdf)
        * [2022](https://web.archive.org/web/20221018201001/https://www.farmaciavaticana.va/images/pdf/calendario_2022.pdf)
        * [2023](https://web.archive.org/web/20231101221620/https://www.farmaciavaticana.va/images/pdf/calendario_2023.pdf)
        * [2024](https://web.archive.org/web/20241208034527/https://www.farmaciavaticana.va/media/attachments/2024/01/02/calendario_2024.pdf)
        * [2025](https://web.archive.org/web/20250208160702/https://www.farmaciavaticana.va/media/attachments/2025/01/02/calendario-2025.pdf)
    """

    country = "VA"
    default_language = "it"
    supported_languages = ("en_US", "it", "th")
    # Lateran Treaty, FEB 11, 2029.
    start_year = 1929

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self) -> None:
        # This is supposedly the same as International New Year.
        # Modern adoption across the entire Latin Church in 1931 though this
        # was already celebrated in Rome as the Octave day of Christmas.

        # Solemnity of Mary, Mother of God.
        self._add_holiday_jan_1(tr("Solennità di Maria Santissima Madre di Dio"))

        # Epiphany.
        self._add_epiphany_day(tr("Epifania del Signore"))

        self._add_holiday_feb_11(
            # Anniversary of the Foundation of Vatican City.
            tr("Anniversario della istituzione dello Stato della Città del Vaticano")
        )

        # Name Day of the Holy Father.
        name_day = tr("Onomastico del Santo Padre")

        if self._year >= 2025:
            # Pope Leo XIV (Robert Francis Prevost).
            # Name Day: Saint Robert Bellarmine Day (SEP 17).
            self._add_holiday_sep_17(name_day)

            if self._year == 2025:
                # Pope Francis (cont.).
                self._add_saint_georges_day(name_day)
        elif self._year >= 2013:
            # Pope Francis (Jorge Mario Bergoglio).
            # Name Day: Saint George's Day (APR 23).
            self._add_saint_georges_day(name_day)
        elif self._year >= 2006:
            # Pope Benedict XVI (Josef Aloisius Ratzinger).
            # Name Day: Saint Joseph's Day (MAR 19).
            self._add_saint_josephs_day(name_day)
        elif 1978 <= self._year <= 2004:
            # Pope John Paul II (Karol Józef Wojtyła).
            # Name Day: Saint Charles Borromeo Day (NOV 4).
            self._add_holiday_nov_4(name_day)

        # Anniversary of the Election of the Holy Father.
        name_election = tr("Anniversario dell'Elezione del Santo Padre")

        if self._year >= 2026:
            # Pope Leo XIV (Robert Francis Prevost).
            self._add_holiday_may_8(name_election)
        elif self._year >= 2014:
            # Pope Francis (Jorge Mario Bergoglio).
            self._add_holiday_mar_13(name_election)
        elif 2006 <= self._year <= 2012:
            # Pope Benedict XVI (Josef Aloisius Ratzinger).
            self._add_holiday_apr_19(name_election)
        elif 1979 <= self._year <= 2004:
            # Pope John Paul II (Karol Józef Wojtyła).
            self._add_holiday_oct_16(name_election)
        elif 1964 <= self._year <= 1978:
            # Pope Paul VI (Giovanni Battista Enrico Antonio Maria Montini).
            self._add_holiday_jun_21(name_election)
        elif 1959 <= self._year <= 1962:
            # Pope John XXIII (Angelo Giuseppe Roncalli).
            self._add_holiday_oct_28(name_election)
        elif 1940 <= self._year <= 1958:
            # Pope Pius XII (Eugenio Maria Giuseppe Giovanni Pacelli).
            self._add_holiday_mar_2(name_election)
        elif self._year <= 1939:
            # Pope Pius XI (Achille Ambrogio Damiano Ratti).
            self._add_holiday_feb_6(name_election)

        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("San Giuseppe"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Giovedì Santo"))

        # Good Friday.
        self._add_good_friday(tr("Venerdì Santo"))

        # Holy Saturday.
        self._add_holy_saturday(tr("Sabato Santo"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Pasqua di Resurrezione"))

        # Easter Monday.
        self._add_easter_monday(tr("Lunedì dell'Angelo"))

        # Easter Tuesday.
        self._add_easter_tuesday(tr("Martedì in Albis"))

        # Created in response to May Day holidays by Pope Pius XII in 1955.
        if self._year >= 1955:
            # Saint Joseph the Worker.
            self._add_holiday_may_1(tr("San Giuseppe Artigiano"))

        # Solemnity of Pentecost.
        self._add_whit_sunday(tr("Solennità della Pentecoste"))

        # Solemnity of Holy Trinity.
        self._add_trinity_sunday(tr("Solennità della Santissima Trinità"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Ascensione del Signore"))

        # Corpus Domini.
        self._add_corpus_christi_day(tr("Corpus Domini"))

        # Saints Peter and Paul's Day.
        self._add_saints_peter_and_paul_day(tr("Santi Pietro e Paolo"))

        # Day Before Assumption of Mary.
        self._add_holiday_aug_14(tr("Vigilia dell'Assunzione di Maria Santissima"))

        # Assumption of Mary Day.
        self._add_assumption_of_mary_day(tr("Assunzione di Maria Santissima"))

        # Day After Assumption of Mary.
        self._add_holiday_aug_16(tr("Giorno Successivo all'Assunzione di Maria Santissima"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Tutti i Santi"))

        # All Souls' Day.
        self._add_all_souls_day(tr("Tutti i Fedeli Defunti"))

        # Immaculate Conception.
        self._add_immaculate_conception_day(tr("Immacolata Concezione"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Vigilia di Natale"))

        # Christmas Day.
        self._add_christmas_day(tr("Natale"))

        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Santo Stefano"))

        # Saint John the Evangelist's Day.
        self._add_christmas_day_three(tr("San Giovanni"))

        # Last Day of the Year.
        self._add_new_years_eve(tr("Ultimo giorno dell'anno"))


class VA(VaticanCity):
    pass


class VAT(VaticanCity):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import NOV
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Poland(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Poland holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Poland>
        * <https://pl.wikipedia.org/wiki/Dni_wolne_od_pracy_w_Polsce>
        * <https://web.archive.org/web/20250402103635/https://isap.sejm.gov.pl/isap.nsf/DocDetails.xsp?id=WDU20240001965>
    """

    country = "PL"
    default_language = "pl"
    supported_languages = ("de", "en_US", "pl", "uk")
    start_year = 1925

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, PolandStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nowy Rok"))

        if self._year <= 1960 or self._year >= 2011:
            # Epiphany.
            self._add_epiphany_day(tr("Święto Trzech Króli"))

        if self._year <= 1950:
            # Candlemas.
            self._add_candlemas(tr("Oczyszczenie Najświętszej Marii Panny"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Niedziela Wielkanocna"))

        # Easter Monday.
        self._add_easter_monday(tr("Poniedziałek Wielkanocny"))

        if self._year >= 1950:
            # National Day.
            self._add_holiday_may_1(tr("Święto Państwowe"))

        if self._year <= 1950 or self._year >= 1990:
            # National Day of the Third of May.
            self._add_holiday_may_3(tr("Święto Narodowe Trzeciego Maja"))

        if 1946 <= self._year <= 1950:
            # National Victory and Freedom Day.
            self._add_holiday_may_9(tr("Narodowe Święto Zwycięstwa i Wolności"))

        if self._year <= 1950:
            # Ascension Day.
            self._add_ascension_thursday(tr("Wniebowstąpienie Pańskie"))

        # Pentecost.
        self._add_whit_sunday(tr("Zielone Świątki"))

        if self._year <= 1950:
            # Whit Monday.
            self._add_whit_monday(tr("Drugi dzień Zielonych Świątek"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Dzień Bożego Ciała"))

        if self._year <= 1950:
            self._add_saints_peter_and_paul_day(
                # Saints Peter and Paul Day.
                tr("Uroczystość Świętych Apostołów Piotra i Pawła")
            )

        if 1945 <= self._year <= 1989:
            # National Day of Rebirth of Poland.
            self._add_holiday_jul_22(tr("Narodowe Święto Odrodzenia Polski"))

        if self._year <= 1960 or self._year >= 1989:
            # Assumption Day.
            self._add_assumption_of_mary_day(tr("Wniebowzięcie Najświętszej Marii Panny"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Uroczystość Wszystkich Świętych"))

        if 1937 <= self._year <= 1944 or self._year >= 1989:
            # National Independence Day.
            self._add_holiday_nov_11(tr("Narodowe Święto Niepodległości"))

        if self._year <= 1950:
            self._add_immaculate_conception_day(
                # Immaculate Conception of the Blessed Virgin Mary.
                tr("Niepokalane Poczęcie Najświętszej Marii Panny")
            )

        if self._year >= 2025:
            # Christmas Eve.
            self._add_christmas_eve(tr("Wigilia Bożego Narodzenia"))

        # Christmas Day.
        self._add_christmas_day(tr("Boże Narodzenie (pierwszy dzień)"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Boże Narodzenie (drugi dzień)"))


class PL(Poland):
    pass


class POL(Poland):
    pass


class PolandStaticHolidays:
    special_public_holidays = {
        # National Independence Day - 100th anniversary.
        2018: (NOV, 12, tr("Narodowe Święto Niepodległości - 100-lecie")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomHinduHolidays
from holidays.calendars.gregorian import SEP, OCT
from holidays.groups import HinduCalendarHolidays, TibetanCalendarHolidays
from holidays.holiday_base import HolidayBase


class Bhutan(HolidayBase, HinduCalendarHolidays, TibetanCalendarHolidays):
    """Bhutan holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Bhutan>
        * [The Bhutanese calendar](https://web.archive.org/web/20250707105447/http://www.kalacakra.org/calendar/bhutlist.htm)
        * [2007-2011](https://web.archive.org/web/20070730055559/http://www.rcsc.gov.bt/tmpFolder/CalendarOfEvent/holiday.htm)
        * [2010-2011](https://web.archive.org/web/20100127021618/http://www.mohca.gov.bt/?mode=READMORE&news_id=157)
        * [2014](https://web.archive.org/web/20141222122050/http://www.mohca.gov.bt/Publications/calander_2014-15.pdf)
        * [2015](https://web.archive.org/web/20151004161522/http://www.mohca.gov.bt/Publications/calander_2015-16.pdf)
        * [2016](https://web.archive.org/web/20170606131500/http://www.mohca.gov.bt/download/calander_2016-17.pdf)
        * [2017](https://web.archive.org/web/20180107115733/http://www.mohca.gov.bt/wp-content/uploads/2017/11/calander_2017-18.pdf)
        * [2018](https://web.archive.org/web/20180906031327/http://www.mohca.gov.bt/wp-content/uploads/2017/11/scan0126.pdf)
        * [2019](https://web.archive.org/web/20191221201614/http://www.mohca.gov.bt/downloads/scan0004.pdf)
        * [2020](https://web.archive.org/web/20200720030220/http://www.mohca.gov.bt/wp-content/uploads/2019/12/2020-Holiday-List0001.pdf)
        * [2021](https://web.archive.org/web/20230511011459/http://www.mohca.gov.bt/downloads/Holidays2021.pdf)
        * [2022](https://web.archive.org/web/20220619061040/https://www.mohca.gov.bt/wp-content/uploads/2021/11/HolidayList.pdf)
        * [2023](https://web.archive.org/web/20221210152039/https://www.mohca.gov.bt/wp-content/uploads/2022/11/Holidays-2023.pdf)
        * [2024](https://web.archive.org/web/20250407140158/https://www.moha.gov.bt/wp-content/uploads/2023/11/National-Holiday-List-for-2024-2-.pdf)
        * [2025](https://web.archive.org/web/20250703030226/https://www.moha.gov.bt/wp-content/uploads/2024/11/Calendar_2025.pdf)
    """

    country = "BT"
    default_language = "dz"
    # %s (estimated).
    estimated_label = tr("%s (ཚོད་དཔག་གི།)")
    supported_languages = ("dz", "en_US")
    # Jigme Khesar Namgyel Wangchuck ascended to the throne on December 9th, 2006.
    start_year = 2007
    subdivisions = (
        "11",  # Paro.
        "12",  # Chhukha.
        "13",  # Haa.
        "14",  # Samtse.
        "15",  # Thimphu.
        "21",  # Tsirang.
        "22",  # Dagana.
        "23",  # Punakha.
        "24",  # Wangdue Phodrang.
        "31",  # Sarpang.
        "32",  # Trongsa.
        "33",  # Bumthang.
        "34",  # Zhemgang.
        "41",  # Trashigang.
        "42",  # Monggar.
        "43",  # Pema Gatshel.
        "44",  # Lhuentse.
        "45",  # Samdrup Jongkhar.
        "GA",  # Gasa.
        "TY",  # Trashi Yangtse.
    )

    subdivisions_aliases = {
        "Paro": "11",
        "Chhukha": "12",
        "Haa": "13",
        "Samtse": "14",
        "Thimphu": "15",
        "Tsirang": "21",
        "Dagana": "22",
        "Punakha": "23",
        "Wangdue Phodrang": "24",
        "Sarpang": "31",
        "Trongsa": "32",
        "Bumthang": "33",
        "Zhemgang": "34",
        "Trashigang": "41",
        "Monggar": "42",
        "Pema Gatshel": "43",
        "Lhuentse": "44",
        "Samdrup Jongkhar": "45",
        "Gasa": "GA",
        "Trashi Yangtse": "TY",
    }

    def __init__(self, *args, hindu_show_estimated: bool = True, **kwargs):
        """
        Args:
            hindu_show_estimated:
                Whether to add "estimated" label to Hindu holidays name
                if holiday date is estimated.
        """
        HinduCalendarHolidays.__init__(
            self, cls=BhutanHinduHolidays, show_estimated=hindu_show_estimated
        )
        TibetanCalendarHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Birth Anniversary of His Majesty the King.
        name = tr("མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།")
        self._add_holiday_feb_21(name)
        self._add_holiday_feb_22(name)
        self._add_holiday_feb_23(name)

        # Birth Anniversary of the 3rd Druk Gyalpo.
        self._add_holiday_may_2(tr("མི་དབང་འབྲུག་རྒྱལ་གསུམ་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"))

        # Coronation of His Majesty the King.
        self._add_holiday_nov_1(tr("མི་དབང་མངའ་བདག་རིན་པོ་ཆེའི་གསེར་ཁྲི་མངའ་གསོལ་གྱི་དུས་ཆེན་ངལ་གསོལ།"))

        # Birth Anniversary of the 4th Druk Gyalpo - Constitution Day.
        self._add_holiday_nov_11(tr("མི་དབང་འབྲུག་རྒྱལ་བཞི་པའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོལ།"))

        # National Day.
        self._add_holiday_dec_17(tr("རྒྱལ་ཡོངས་དུས་ཆེན་གྱི་ངལ་གསོལ།"))

        # Winter Solstice.
        self._add_tibetan_winter_solstice(tr("དགུན་ཉི་ལྡོག་གི་ངལ་གསོལ།"))

        # Traditional Day of Offering.
        self._add_day_of_offering(tr("སྔར་སྲོལ་འབུལ་བའི་ལོ་གསར་གྱི་ངལ་གསོལ།"))

        # Losar.
        name = tr("གནམ་ལོ་གསར་ཚེས་ཀྱིས་དུས་སྟོན་ངལ་གསོལ།")
        self._add_losar(name)
        self._add_losar_day_two(name)

        # Death Anniversary of Zhabdrung.
        self._add_death_of_zhabdrung(tr("མཐུ་ཆེན་ཆོས་ཀྱི་རྒྱལ་པོའི་དུས་ཆེན་ངལ་གསོལ།"))

        # Lord Buddha's Parinirvana.
        self._add_buddha_parinirvana(tr("སངས་རྒྱས་བཅོམ་ལྡན་འདས་ཀྱི་དུས་ཆེན་ལྔ་འཛོམས་ངལ་གསོལ།"))

        # Birth Anniversary of Guru Rinpoche.
        self._add_birth_of_guru_rinpoche(tr("ཨོ་རྒྱན་གུ་རུ་རིན་པོ་ཆེའི་འཁྲུངས་སྐར་དུས་ཆེན་ངལ་གསོ།"))

        # First Sermon of Lord Buddha.
        self._add_buddha_first_sermon(tr("སྟོན་པ་མཆོག་ཝ་ར་ན་སིར་བདེན་བཞིའི་ཆོས་འཁོར་བསྐོར་བའི་དུས་ཆེན་ངལ་གསོ།"))

        # Blessed Rainy Day.
        self._add_blessed_rainy_day(tr("རྒྱལ་ཡོངས་ཁྲུས་བབས་ངལ་གསོ།"))

        # Dassain.
        self._add_dussehra(tr("ལྷོ་མཚམས་ད་སའིན་ངལ་གསོ།"))

        # Descending Day of Lord Buddha.
        self._add_descending_day_of_lord_buddha(tr("རྒྱལ་བ་ལྷ་ལས་བབས་པའི་དུས་ཆེན་ངལ་གསོ།"))

    def _populate_subdiv_15_public_holidays(self):
        # Thimphu Drubchoe.
        self._add_thimphu_drubchen_day(tr("ཐིམ་རྫོང་ལྷ་མོའི་དངོས་འཆམ་མཇལ་རྒྱུའི་ངལ་གསོ།"))

        # Thimphu Tshechu.
        name = tr("ཐིམ་ཕུ་ཚེས་བཅུའི་ངལ་གསོ།")
        self._add_thimphu_tshechu_day(name)
        self._add_thimphu_tshechu_day_two(name)
        self._add_thimphu_tshechu_day_three(name)


class BT(Bhutan):
    pass


class BTN(Bhutan):
    pass


class BhutanHinduHolidays(_CustomHinduHolidays):
    DUSSEHRA_DATES = {
        2007: (OCT, 21),
        2008: (OCT, 9),
        2009: (SEP, 28),
        2010: (OCT, 17),
        2011: (OCT, 6),
        2014: (OCT, 3),
        2015: (OCT, 22),
        2016: (OCT, 11),
        2017: (SEP, 30),
        2018: (OCT, 19),
        2019: (OCT, 8),
        2020: (OCT, 26),
        2021: (OCT, 15),
        2022: (OCT, 5),
        2023: (OCT, 24),
        2024: (OCT, 12),
        2025: (OCT, 2),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Malawi(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Malawi holidays.

    References:
        * <https://web.archive.org/web/20250414071022/https://www.officeholidays.com/countries/malawi>
        * <https://web.archive.org/web/20250213080059/https://www.timeanddate.com/holidays/malawi/>
    """

    country = "MW"
    observed_label = "%s (observed)"
    start_year = 2000

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        self._add_observed(self._add_new_years_day("New Year's Day"))

        self._add_observed(self._add_holiday_jan_15("John Chilembwe Day"))

        self._add_observed(self._add_holiday_mar_3("Martyrs Day"))

        self._add_good_friday("Good Friday")

        self._add_easter_monday("Easter Monday")

        self._add_observed(self._add_labor_day("Labour Day"))

        self._add_observed(self._add_holiday_may_14("Kamuzu Day"))

        self._add_observed(self._add_holiday_jul_6("Independence Day"))

        self._add_observed(self._add_holiday_oct_15("Mother's Day"))

        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        self._add_observed(self._add_christmas_day_two("Boxing Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)


class MW(Malawi):
    pass


class MWI(Malawi):
    pass
